#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: iosxr_logging
author:
- Trishna Guha (@trishnaguha)
- Kedar Kekan (@kedarX)
short_description: (deprecated, removed after 2023-08-01) Configuration management of system logging services on network
  devices
description:
- This module provides declarative management configuration of system logging (syslog)
  on Cisco IOS XR devices.
version_added: 1.0.0
deprecated:
  alternative: iosxr_logging_global
  why: Updated module released with more functionality.
  removed_at_date: '2023-08-01'
requirements:
- ncclient >= 0.5.3 when using netconf
- lxml >= 4.1.1 when using netconf
notes:
- This module works with connection C(network_cli) and C(netconf). See L(the IOS-XR
  Platform Options,../network/user_guide/platform_iosxr.html).
options:
  dest:
    description:
    - Destination for system logging (syslog) messages.
    choices:
    - host
    - console
    - monitor
    - buffered
    - file
    type: str
  name:
    description:
    - When C(dest) = I(file) name indicates file-name
    - When C(dest) = I(host) name indicates the host-name or ip-address of syslog
      server.
    type: str
  vrf:
    description:
    - vrf name when syslog server is configured, C(dest) = C(host)
    type: str
    default: default
  size:
    description:
    - Size of buffer when C(dest) = C(buffered). The acceptable value is in the range
      I(307200 to 125000000 bytes). Default 307200
    - Size of file when C(dest) = C(file). The acceptable value is in the range I(1
      to 2097152)KB. Default 2 GB
    type: int
  facility:
    description:
    - To configure the type of syslog facility in which system logging (syslog) messages
      are sent to syslog servers Optional config for C(dest) = C(host)
    default: local7
    type: str
  hostnameprefix:
    description:
    - To append a hostname prefix to system logging (syslog) messages logged to syslog
      servers. Optional config for C(dest) = C(host)
    type: str
  level:
    description:
    - Specifies the severity level for the logging.
    type: str
    default: debugging
    aliases:
    - severity
    choices: ["emergencies", "alerts", "critical", "errors", "warning", "notifications", "informational", "debugging"]
  path:
    description:
        Set file path.
    type: str
  aggregate:
    description: List of syslog logging configuration definitions.
    type: list
    elements: dict
    suboptions:
      dest:
        description:
        - Destination for system logging (syslog) messages.
        choices:
        - host
        - console
        - monitor
        - buffered
        - file
        type: str
      name:
        description:
        - When C(dest) = I(file) name indicates file-name
        - When C(dest) = I(host) name indicates the host-name or ip-address of syslog
          server.
        type: str
      path:
        description:
          Set file path.
        type: str
      vrf:
        description:
        - vrf name when syslog server is configured, C(dest) = C(host)
        type: str
      size:
        description:
        - Size of buffer when C(dest) = C(buffered). The acceptable value is in the range
          I(307200 to 125000000 bytes). Default 307200
        - Size of file when C(dest) = C(file). The acceptable value is in the range I(1
          to 2097152)KB. Default 2 GB
        type: int
      facility:
        description:
        - To configure the type of syslog facility in which system logging (syslog) messages
          are sent to syslog servers Optional config for C(dest) = C(host)
        type: str
      hostnameprefix:
        description:
        - To append a hostname prefix to system logging (syslog) messages logged to syslog
          servers. Optional config for C(dest) = C(host)
        type: str
      level:
        description:
        - Specifies the severity level for the logging.
        type: str
        aliases:
        - severity
        choices: ["emergencies", "alerts", "critical", "errors", "warning", "notifications", "informational", "debugging"]
      state:
        description:
        - Existential state of the logging configuration on the node.
        choices:
        - present
        - absent
        type: str
  state:
    description:
    - Existential state of the logging configuration on the node.
    default: present
    choices:
    - present
    - absent
    type: str
extends_documentation_fragment:
- cisco.iosxr.iosxr
"""

EXAMPLES = """
- name: configure logging for syslog server host
  cisco.iosxr.iosxr_logging:
    dest: host
    name: 10.10.10.1
    level: critical
    state: present

- name: add hostnameprefix configuration
  cisco.iosxr.iosxr_logging:
    hostnameprefix: host1
    state: absent

- name: add facility configuration
  cisco.iosxr.iosxr_logging:
    facility: local1
    state: present

- name: configure console logging level
  cisco.iosxr.iosxr_logging:
    dest: console
    level: debugging
    state: present

- name: configure monitor logging level
  cisco.iosxr.iosxr_logging:
    dest: monitor
    level: errors
    state: present

- name: configure syslog to a file
  cisco.iosxr.iosxr_logging:
    dest: file
    name: file_name
    size: 2048
    level: errors
    state: present

- name: configure buffered logging with size
  cisco.iosxr.iosxr_logging:
    dest: buffered
    size: 5100000

- name: Configure logging using aggregate
  cisco.iosxr.iosxr_logging:
    aggregate:
      - {dest: console, level: warning}
      - {dest: buffered, size: 4800000}
      - {dest: file, name: file3, size: 2048}
      - {dest: host, name: host3, level: critical}

- name: Delete logging using aggregate
  cisco.iosxr.iosxr_logging:
    aggregate:
      - {dest: console, level: warning}
      - {dest: buffered, size: 4800000}
      - {dest: file, name: file3, size: 2048}
      - {dest: host, name: host3, level: critical}
    state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always (empty list when no commands to send)
  type: list
  sample:
    - logging 10.10.10.1 vrf default severity debugging
    - logging facility local7
    - logging hostnameprefix host1
    - logging console critical
    - logging buffered 2097153
    - logging buffered warnings
    - logging monitor errors
    - logging file log_file maxfilesize 1024 severity info
xml:
  description: NetConf rpc xml sent to device with transport C(netconf)
  returned: always (empty list when no xml rpc to send)
  type: list
  sample:
    - '<config xmlns:xc="urn:ietf:params:xml:ns:netconf:base:1.0">
    <syslog xmlns="http://cisco.com/ns/yang/Cisco-IOS-XR-infra-syslog-cfg">
    <files>
    <file xc:operation="delete">
    <file-name>file1</file-name>
    <file-log-attributes>
    <max-file-size>2097152</max-file-size>
    <severity>2</severity>
    </file-log-attributes>
    </file>
    </files>
    </syslog>
    </config>'
"""

import collections
import re

from copy import deepcopy

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    remove_default_spec,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.iosxr import (
    build_xml,
    etree_find,
    etree_findall,
    get_capabilities,
    get_config,
    get_os_version,
    is_cliconf,
    is_netconf,
    load_config,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.utils.utils import Version


severity_level = {
    "emergency": "0",
    "alert": "1",
    "critical": "2",
    "error": "3",
    "warning": "4",
    "notice": "5",
    "info": "6",
    "debug": "7",
    "disable": "15",
}

severity_transpose = {
    "emergencies": "emergency",
    "alerts": "alert",
    "critical": "critical",
    "errors": "error",
    "warning": "warning",
    "notifications": "notice",
    "informational": "info",
    "debugging": "debug",
}


class ConfigBase(object):
    def __init__(self, module):
        self._flag = None
        self._module = module
        self._result = {"changed": False, "warnings": []}
        self._want = list()
        self._have = list()

    def validate_size(self, value, type=None):
        if value:
            if type == "buffer":
                if value and not int(307200) <= value <= int(125000000):
                    self._module.fail_json(
                        msg="buffer size must be between 307200 and 125000000",
                    )
            elif type == "file":
                if value and not int(1) <= value <= int(2097152):
                    self._module.fail_json(
                        msg="file size must be between 1 and 2097152",
                    )
        return value

    def map_params_to_obj(self, required_if=None):
        aggregate = self._module.params.get("aggregate")
        if aggregate:
            for item in aggregate:
                for key in item:
                    if item.get(key) is None:
                        item[key] = self._module.params[key]

                d = item.copy()

                if d["dest"] not in ("host", "file"):
                    d["name"] = None

                if d["dest"] == "buffered":
                    if d["size"] is not None:
                        d["size"] = str(
                            self.validate_size(d["size"], "buffer"),
                        )
                    else:
                        d["size"] = str(307200)
                elif d["dest"] == "file":
                    if d["size"] is not None:
                        d["size"] = str(self.validate_size(d["size"], "file"))
                    else:
                        d["size"] = str(2097152)
                else:
                    d["size"] = None

                if self._flag == "NC":
                    d["level"] = severity_transpose[d["level"]]

                self._want.append(d)

        else:
            params = self._module.params
            if params["dest"] not in ("host", "file"):
                params["name"] = None

            if params["dest"] == "buffered":
                if params["size"] is not None:
                    params["size"] = str(
                        self.validate_size(params["size"], "buffer"),
                    )
                else:
                    params["size"] = str(307200)
            elif params["dest"] == "file":
                if params["size"] is not None:
                    params["size"] = str(
                        self.validate_size(params["size"], "file"),
                    )
                else:
                    params["size"] = str(2097152)
            else:
                params["size"] = None

            if self._flag == "NC":
                params["level"] = severity_transpose[params["level"]]

            self._want.append(
                {
                    "dest": params["dest"],
                    "name": params["name"],
                    "vrf": params["vrf"],
                    "size": params["size"],
                    "path": params["path"],
                    "facility": params["facility"],
                    "level": params["level"],
                    "hostnameprefix": params["hostnameprefix"],
                    "state": params["state"],
                },
            )


class CliConfiguration(ConfigBase):
    def __init__(self, module):
        super(CliConfiguration, self).__init__(module)
        self._file_list = set()
        self._host_list = set()

    def map_obj_to_commands(self, os_version):
        commands = list()
        for want_item in self._want:
            dest = want_item["dest"]
            name = want_item["name"]
            size = want_item["size"]
            path = want_item["path"]
            facility = want_item["facility"]
            level = want_item["level"]
            vrf = want_item["vrf"]
            hostnameprefix = want_item["hostnameprefix"]
            state = want_item["state"]
            del want_item["state"]

            have_size = None
            have_console_level = None
            have_monitor_level = None
            have_prefix = None
            have_facility = None

            for item in self._have:
                if item["dest"] == "buffered":
                    have_size = item["size"]
                if item["dest"] == "console":
                    have_console_level = item["level"]
                if item["dest"] == "monitor":
                    have_monitor_level = item["level"]
                if item["dest"] is None and item["hostnameprefix"] is not None:
                    have_prefix = item["hostnameprefix"]
                if (
                    item["dest"] is None
                    and item["hostnameprefix"] is None
                    and item["facility"] is not None
                ):
                    have_facility = item["facility"]

            if state == "absent":
                if dest == "host" and name in self._host_list:
                    commands.append("no logging {0} vrf {1}".format(name, vrf))
                elif dest == "file" and name in self._file_list:
                    commands.append("no logging file {0}".format(name))
                elif dest == "console" and have_console_level is not None:
                    commands.append("no logging {0}".format(dest))
                elif dest == "monitor" and have_monitor_level:
                    commands.append("no logging {0}".format(dest))
                elif dest == "buffered" and have_size:
                    commands.append("no logging {0}".format(dest))

                if dest is None and hostnameprefix is not None and have_prefix == hostnameprefix:
                    commands.append(
                        "no logging hostnameprefix {0}".format(hostnameprefix),
                    )
                if dest is None and facility is not None and have_facility == facility:
                    commands.append("no logging facility {0}".format(facility))

            if state == "present":
                if dest == "host" and name not in self._host_list:
                    if level == "errors" or level == "informational":
                        level = severity_transpose[level]
                    commands.append(
                        "logging {0} vrf {1} severity {2}".format(
                            name,
                            vrf,
                            level,
                        ),
                    )
                elif dest == "file" and name not in self._file_list:
                    if level == "errors" or level == "informational":
                        level = severity_transpose[level]
                    if os_version and Version(os_version) > Version("7.0"):
                        commands.append(
                            "logging file {0} path {1} maxfilesize {2} severity {3}".format(
                                name,
                                path,
                                size,
                                level,
                            ),
                        )
                    else:
                        commands.append(
                            "logging file {0} maxfilesize {1} severity {2}".format(
                                name,
                                size,
                                level,
                            ),
                        )
                elif dest == "buffered" and (
                    have_size is None or (have_size is not None and size != have_size)
                ):
                    commands.append("logging buffered {0}".format(size))
                elif dest == "console" and (
                    have_console_level is None
                    or (have_console_level is not None and have_console_level != level)
                ):
                    commands.append("logging console {0}".format(level))
                elif dest == "monitor" and (
                    have_monitor_level is None
                    or (have_monitor_level is not None and have_monitor_level != level)
                ):
                    commands.append("logging monitor {0}".format(level))

                if (
                    dest is None
                    and hostnameprefix is not None
                    and (
                        have_prefix is None
                        or (have_prefix is not None and hostnameprefix != have_prefix)
                    )
                ):
                    commands.append(
                        "logging hostnameprefix {0}".format(hostnameprefix),
                    )
                if dest is None and hostnameprefix is None and facility != have_facility:
                    commands.append("logging facility {0}".format(facility))

        self._result["commands"] = commands
        if commands:
            commit = not self._module.check_mode
            diff = load_config(self._module, commands, commit=commit)
            if diff:
                self._result["diff"] = dict(prepared=diff)
            self._result["changed"] = True

    def parse_facility(self, line):
        match = re.search(r"logging facility (\S+)", line, re.M)
        facility = None
        if match:
            facility = match.group(1)

        return facility

    def parse_size(self, line, dest):
        size = None

        if dest == "buffered":
            match = re.search(r"logging buffered (\S+)", line, re.M)
            if match:
                try:
                    int_size = int(match.group(1))
                except ValueError:
                    int_size = None

                if int_size is not None:
                    if isinstance(int_size, int):
                        size = str(match.group(1))
        if dest == "file":
            match = re.search(
                r"logging file (\S+) (path\s\S+\s)?maxfilesize (\S+)",
                line,
                re.M,
            )
            if match:
                try:
                    if "path" in line:
                        int_size = int(match.group(2))
                    else:
                        int_size = int(match.group(1))
                except ValueError:
                    int_size = None

                if int_size is not None:
                    if isinstance(int_size, int):
                        size = str(int_size)

        return size

    def parse_path(self, line, dest):
        path = None

        if dest == "file":
            match = re.search(r"logging file (\S+) (path\s\S+\s)", line, re.M)
            if match:
                try:
                    path = to_text(
                        match.group(2),
                        errors="surrogate_or_strict",
                    )
                except ValueError:
                    path = None

        return path

    def parse_hostnameprefix(self, line):
        prefix = None
        match = re.search(r"logging hostnameprefix (\S+)", line, re.M)
        if match:
            prefix = match.group(1)
        return prefix

    def parse_name(self, line, dest):
        name = None
        if dest == "file":
            match = re.search(r"logging file (\S+)", line, re.M)
            if match:
                name = match.group(1)
        elif dest == "host":
            match = re.search(r"logging (\S+)", line, re.M)
            if match:
                name = match.group(1)

        return name

    def parse_level(self, line, dest):
        level_group = (
            "emergencies",
            "alerts",
            "critical",
            "errors",
            "warning",
            "notifications",
            "informational",
            "debugging",
        )

        level = None
        match = re.search(r"logging {0} (\S+)".format(dest), line, re.M)
        if match:
            if match.group(1) in level_group:
                level = match.group(1)

        return level

    def parse_dest(self, line, group):
        dest_group = ("console", "monitor", "buffered", "file")
        dest = None
        if group in dest_group:
            dest = group
        elif "vrf" in line:
            dest = "host"

        return dest

    def parse_vrf(self, line, dest):
        vrf = None
        if dest == "host":
            match = re.search(r"logging (\S+) vrf (\S+)", line, re.M)
            if match:
                vrf = match.group(2)
        return vrf

    def map_config_to_obj(self):
        data = get_config(self._module, config_filter="logging")
        lines = data.split("\n")

        for line in lines:
            match = re.search(r"logging (\S+)", line, re.M)
            if match:
                dest = self.parse_dest(line, match.group(1))
                name = self.parse_name(line, dest)
                if dest == "host" and name is not None:
                    self._host_list.add(name)
                if dest == "file" and name is not None:
                    self._file_list.add(name)

                self._have.append(
                    {
                        "dest": dest,
                        "name": name,
                        "size": self.parse_size(line, dest),
                        "path": self.parse_path(line, dest),
                        "facility": self.parse_facility(line),
                        "level": self.parse_level(line, dest),
                        "vrf": self.parse_vrf(line, dest),
                        "hostnameprefix": self.parse_hostnameprefix(line),
                    },
                )

    def run(self, os_version):
        self.map_params_to_obj()
        self.map_config_to_obj()
        self.map_obj_to_commands(os_version)

        return self._result


class NCConfiguration(ConfigBase):
    def __init__(self, module):
        super(NCConfiguration, self).__init__(module)
        self._flag = "NC"
        self._log_file_meta = collections.OrderedDict()
        self._log_host_meta = collections.OrderedDict()
        self._log_console_meta = collections.OrderedDict()
        self._log_monitor_meta = collections.OrderedDict()
        self._log_buffered_meta = collections.OrderedDict()
        self._log_facility_meta = collections.OrderedDict()
        self._log_prefix_meta = collections.OrderedDict()

    def map_obj_to_xml_rpc(self, os_version):
        file_attribute_path = "file-log-attributes"
        if os_version and Version(os_version) > Version("7.0.0"):
            file_attribute_path = "file-specification"
            self._log_file_meta.update(
                [
                    (
                        "files",
                        {
                            "xpath": "syslog/files",
                            "tag": True,
                            "operation": "edit",
                        },
                    ),
                    (
                        "file",
                        {
                            "xpath": "syslog/files/file",
                            "tag": True,
                            "operation": "edit",
                            "attrib": "operation",
                        },
                    ),
                    (
                        "a:name",
                        {
                            "xpath": "syslog/files/file/file-name",
                            "operation": "edit",
                        },
                    ),
                    (
                        "file-attrib",
                        {
                            "xpath": "syslog/files/file/" + file_attribute_path,
                            "tag": True,
                            "operation": "edit",
                        },
                    ),
                    (
                        "a:size",
                        {
                            "xpath": "syslog/files/file/" + file_attribute_path + "/max-file-size",
                            "operation": "edit",
                        },
                    ),
                    (
                        "a:level",
                        {
                            "xpath": "syslog/files/file/" + file_attribute_path + "/severity",
                            "operation": "edit",
                        },
                    ),
                    (
                        "a:path",
                        {
                            "xpath": "syslog/files/file/" + file_attribute_path + "/path",
                            "operation": "edit",
                        },
                    ),
                ],
            )
        else:
            self._log_file_meta.update(
                [
                    (
                        "files",
                        {
                            "xpath": "syslog/files",
                            "tag": True,
                            "operation": "edit",
                        },
                    ),
                    (
                        "file",
                        {
                            "xpath": "syslog/files/file",
                            "tag": True,
                            "operation": "edit",
                            "attrib": "operation",
                        },
                    ),
                    (
                        "a:name",
                        {
                            "xpath": "syslog/files/file/file-name",
                            "operation": "edit",
                        },
                    ),
                    (
                        "file-attrib",
                        {
                            "xpath": "syslog/files/file/" + file_attribute_path,
                            "tag": True,
                            "operation": "edit",
                        },
                    ),
                    (
                        "a:size",
                        {
                            "xpath": "syslog/files/file/" + file_attribute_path + "/max-file-size",
                            "operation": "edit",
                        },
                    ),
                    (
                        "a:level",
                        {
                            "xpath": "syslog/files/file/" + file_attribute_path + "/severity",
                            "operation": "edit",
                        },
                    ),
                ],
            )
        self._log_host_meta.update(
            [
                (
                    "host-server",
                    {
                        "xpath": "syslog/host-server",
                        "tag": True,
                        "operation": "edit",
                    },
                ),
                (
                    "vrfs",
                    {
                        "xpath": "syslog/host-server/vrfs",
                        "tag": True,
                        "operation": "edit",
                    },
                ),
                (
                    "vrf",
                    {
                        "xpath": "syslog/host-server/vrfs/vrf",
                        "tag": True,
                        "operation": "edit",
                    },
                ),
                (
                    "a:vrf",
                    {
                        "xpath": "syslog/host-server/vrfs/vrf/vrf-name",
                        "operation": "edit",
                    },
                ),
                (
                    "ipv4s",
                    {
                        "xpath": "syslog/host-server/vrfs/vrf/ipv4s",
                        "tag": True,
                        "operation": "edit",
                    },
                ),
                (
                    "ipv4",
                    {
                        "xpath": "syslog/host-server/vrfs/vrf/ipv4s/ipv4",
                        "tag": True,
                        "operation": "edit",
                        "attrib": "operation",
                    },
                ),
                (
                    "a:name",
                    {
                        "xpath": "syslog/host-server/vrfs/vrf/ipv4s/ipv4/address",
                        "operation": "edit",
                    },
                ),
                (
                    "ipv4-sev",
                    {
                        "xpath": "syslog/host-server/vrfs/vrf/ipv4s/ipv4/ipv4-severity-port",
                        "tag": True,
                        "operation": "edit",
                    },
                ),
                (
                    "a:level",
                    {
                        "xpath": "syslog/host-server/vrfs/vrf/ipv4s/ipv4/ipv4-severity-port/severity",
                        "operation": "edit",
                    },
                ),
            ],
        )
        self._log_console_meta.update(
            [
                (
                    "a:enable-console",
                    {
                        "xpath": "syslog/enable-console-logging",
                        "operation": "edit",
                        "attrib": "operation",
                    },
                ),
                (
                    "console",
                    {
                        "xpath": "syslog/console-logging",
                        "tag": True,
                        "operation": "edit",
                        "attrib": "operation",
                    },
                ),
                (
                    "a:console-level",
                    {
                        "xpath": "syslog/console-logging/logging-level",
                        "operation": "edit",
                    },
                ),
            ],
        )
        self._log_monitor_meta.update(
            [
                (
                    "monitor",
                    {
                        "xpath": "syslog/monitor-logging",
                        "tag": True,
                        "operation": "edit",
                        "attrib": "operation",
                    },
                ),
                (
                    "a:monitor-level",
                    {
                        "xpath": "syslog/monitor-logging/logging-level",
                        "operation": "edit",
                    },
                ),
            ],
        )
        self._log_buffered_meta.update(
            [
                (
                    "buffered",
                    {
                        "xpath": "syslog/buffered-logging",
                        "tag": True,
                        "operation": "edit",
                        "attrib": "operation",
                    },
                ),
                (
                    "a:size",
                    {
                        "xpath": "syslog/buffered-logging/buffer-size",
                        "operation": "edit",
                    },
                ),
                (
                    "a:level",
                    {
                        "xpath": "syslog/buffered-logging/logging-level",
                        "operation": "edit",
                    },
                ),
            ],
        )
        self._log_facility_meta.update(
            [
                (
                    "facility",
                    {
                        "xpath": "syslog/logging-facilities",
                        "tag": True,
                        "operation": "edit",
                        "attrib": "operation",
                    },
                ),
                (
                    "a:facility",
                    {
                        "xpath": "syslog/logging-facilities/facility-level",
                        "operation": "edit",
                    },
                ),
            ],
        )
        self._log_prefix_meta.update(
            [
                (
                    "a:hostnameprefix",
                    {
                        "xpath": "syslog/host-name-prefix",
                        "operation": "edit",
                        "attrib": "operation",
                    },
                ),
            ],
        )

        state = self._module.params["state"]

        _get_filter = build_xml("syslog", opcode="filter")
        running = get_config(
            self._module,
            source="running",
            config_filter=_get_filter,
        )

        file_ele = etree_findall(running, "file")
        file_list = list()
        if len(file_ele):
            for file in file_ele:
                file_name = etree_find(file, "file-name")
                file_list.append(
                    file_name.text if file_name is not None else None,
                )
        vrf_ele = etree_findall(running, "vrf")
        host_list = list()
        for vrf in vrf_ele:
            host_ele = etree_findall(vrf, "ipv4")
            for host in host_ele:
                host_name = etree_find(host, "address")
                host_list.append(
                    host_name.text if host_name is not None else None,
                )

        console_ele = etree_find(running, "console-logging")
        console_level = (
            etree_find(console_ele, "logging-level") if console_ele is not None else None
        )
        have_console = console_level.text if console_level is not None else None

        monitor_ele = etree_find(running, "monitor-logging")
        monitor_level = (
            etree_find(monitor_ele, "logging-level") if monitor_ele is not None else None
        )
        have_monitor = monitor_level.text if monitor_level is not None else None

        buffered_ele = etree_find(running, "buffered-logging")
        buffered_size = (
            etree_find(buffered_ele, "buffer-size") if buffered_ele is not None else None
        )
        have_buffered = buffered_size.text if buffered_size is not None else None

        facility_ele = etree_find(running, "logging-facilities")
        facility_level = (
            etree_find(facility_ele, "facility-level") if facility_ele is not None else None
        )
        have_facility = facility_level.text if facility_level is not None else None

        prefix_ele = etree_find(running, "host-name-prefix")
        have_prefix = prefix_ele.text if prefix_ele is not None else None

        file_params = list()
        host_params = list()
        console_params = dict()
        monitor_params = dict()
        buffered_params = dict()
        facility_params = dict()
        prefix_params = dict()

        opcode = None
        if state == "absent":
            opcode = "delete"
            for item in self._want:
                if item["dest"] == "file" and item["name"] in file_list:
                    item["level"] = severity_level[item["level"]]
                    file_params.append(item)
                elif item["dest"] == "host" and item["name"] in host_list:
                    item["level"] = severity_level[item["level"]]
                    host_params.append(item)
                elif item["dest"] == "console" and have_console:
                    console_params.update({"console-level": item["level"]})
                elif item["dest"] == "monitor" and have_monitor:
                    monitor_params.update({"monitor-level": item["level"]})
                elif item["dest"] == "buffered" and have_buffered:
                    buffered_params["size"] = str(item["size"]) if item["size"] else None
                    buffered_params["level"] = item["level"] if item["level"] else None
                elif (
                    item["dest"] is None
                    and item["hostnameprefix"] is None
                    and item["facility"] is not None
                    and have_facility
                ):
                    facility_params.update({"facility": item["facility"]})
                elif item["dest"] is None and item["hostnameprefix"] is not None and have_prefix:
                    prefix_params.update(
                        {"hostnameprefix": item["hostnameprefix"]},
                    )
        elif state == "present":
            opcode = "merge"
            for item in self._want:
                if item["dest"] == "file":
                    item["level"] = severity_level[item["level"]]
                    file_params.append(item)
                elif item["dest"] == "host":
                    item["level"] = severity_level[item["level"]]
                    host_params.append(item)
                elif item["dest"] == "console":
                    console_params.update({"console-level": item["level"]})
                elif item["dest"] == "monitor":
                    monitor_params.update({"monitor-level": item["level"]})
                elif item["dest"] == "buffered":
                    buffered_params["size"] = str(item["size"]) if item["size"] else None
                    buffered_params["level"] = item["level"] if item["level"] else None
                elif (
                    item["dest"] is None
                    and item["hostnameprefix"] is None
                    and item["facility"] is not None
                ):
                    facility_params.update({"facility": item["facility"]})
                elif item["dest"] is None and item["hostnameprefix"] is not None:
                    prefix_params.update(
                        {"hostnameprefix": item["hostnameprefix"]},
                    )

        self._result["xml"] = []
        _edit_filter_list = list()
        if opcode:
            if len(file_params):
                _edit_filter_list.append(
                    build_xml(
                        "syslog",
                        xmap=self._log_file_meta,
                        params=file_params,
                        opcode=opcode,
                    ),
                )
            if len(host_params):
                _edit_filter_list.append(
                    build_xml(
                        "syslog",
                        xmap=self._log_host_meta,
                        params=host_params,
                        opcode=opcode,
                    ),
                )
            if len(console_params):
                _edit_filter_list.append(
                    build_xml(
                        "syslog",
                        xmap=self._log_console_meta,
                        params=console_params,
                        opcode=opcode,
                    ),
                )
            if len(monitor_params):
                _edit_filter_list.append(
                    build_xml(
                        "syslog",
                        xmap=self._log_monitor_meta,
                        params=monitor_params,
                        opcode=opcode,
                    ),
                )
            if len(buffered_params):
                _edit_filter_list.append(
                    build_xml(
                        "syslog",
                        xmap=self._log_buffered_meta,
                        params=buffered_params,
                        opcode=opcode,
                    ),
                )
            if len(facility_params):
                _edit_filter_list.append(
                    build_xml(
                        "syslog",
                        xmap=self._log_facility_meta,
                        params=facility_params,
                        opcode=opcode,
                    ),
                )
            if len(prefix_params):
                _edit_filter_list.append(
                    build_xml(
                        "syslog",
                        xmap=self._log_prefix_meta,
                        params=prefix_params,
                        opcode=opcode,
                    ),
                )

            diff = None
            if len(_edit_filter_list):
                commit = not self._module.check_mode
                diff = load_config(
                    self._module,
                    _edit_filter_list,
                    commit=commit,
                    running=running,
                    nc_get_filter=_get_filter,
                )
            if diff:
                if self._module._diff:
                    self._result["diff"] = dict(prepared=diff)

                self._result["xml"] = _edit_filter_list
                self._result["changed"] = True

    def run(self, os_version):
        self.map_params_to_obj()
        self.map_obj_to_xml_rpc(os_version)

        return self._result


def main():
    """main entry point for module execution"""
    element_spec = dict(
        dest=dict(
            type="str",
            choices=["host", "console", "monitor", "buffered", "file"],
        ),
        name=dict(type="str"),
        size=dict(type="int"),
        path=dict(type="str"),
        vrf=dict(type="str", default="default"),
        facility=dict(type="str", default="local7"),
        hostnameprefix=dict(type="str"),
        level=dict(
            type="str",
            default="debugging",
            aliases=["severity"],
            choices=[
                "emergencies",
                "alerts",
                "critical",
                "errors",
                "warning",
                "notifications",
                "informational",
                "debugging",
            ],
        ),
        state=dict(default="present", choices=["present", "absent"]),
    )

    aggregate_spec = deepcopy(element_spec)

    # remove default in aggregate spec, to handle common arguments
    remove_default_spec(aggregate_spec)

    mutually_exclusive = [("dest", "facility", "hostnameprefix")]

    required_if = [
        ("dest", "host", ["name"]),
        ("dest", "file", ["name"]),
        ("dest", "buffered", ["size"]),
        ("dest", "console", ["level"]),
        ("dest", "monitor", ["level"]),
    ]

    argument_spec = dict(
        aggregate=dict(
            type="list",
            elements="dict",
            options=aggregate_spec,
            mutually_exclusive=mutually_exclusive,
            required_if=required_if,
        ),
    )

    argument_spec.update(element_spec)

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        required_if=required_if,
        supports_check_mode=True,
    )
    config_object = None
    if is_cliconf(module):
        config_object = CliConfiguration(module)
        os_version = get_os_version(module)
    elif is_netconf(module):
        config_object = NCConfiguration(module)
        os_version = get_capabilities(module).get("device_info").get("network_os_version")

    if config_object:
        result = config_object.run(os_version)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Rob White (@wimnat)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: glue_crawler
version_added: 4.1.0
short_description: Manage an AWS Glue crawler
description:
  - Manage an AWS Glue crawler. See U(https://aws.amazon.com/glue/) for details.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_glue_crawler).
    The usage did not change.
author:
  - 'Ivan Chekaldin (@ichekaldin)'
options:
  database_name:
    description:
      - The name of the database where results are written.
    type: str
  description:
    description:
      - Description of the crawler being defined.
    type: str
  name:
    description:
      - The name you assign to this crawler definition. It must be unique in your account.
    required: true
    type: str
  recrawl_policy:
    description:
      - A policy that specifies whether to crawl the entire dataset again, or to crawl only folders that were added since the last crawler run.
    suboptions:
      recrawl_behavior:
        description:
          - Specifies whether to crawl the entire dataset again or to crawl only folders that were added since the last crawler run.
          - Supported options are C(CRAWL_EVERYTHING) and C(CRAWL_NEW_FOLDERS_ONLY).
        type: str
    type: dict
  role:
    description:
      - The name or ARN of the IAM role associated with this crawler.
      - Required when I(state=present).
    type: str
  schema_change_policy:
    description:
      - The policy for the crawler's update and deletion behavior.
    suboptions:
      delete_behavior:
        description:
          - Defines the deletion behavior when the crawler finds a deleted object.
          - Supported options are C(LOG), C(DELETE_FROM_DATABASE), and C(DEPRECATE_IN_DATABASE).
        type: str
      update_behavior:
        description:
          - Defines the update behavior when the crawler finds a changed schema..
          - Supported options are C(LOG) and C(UPDATE_IN_DATABASE).
        type: str
    type: dict
  state:
    description:
      - Create or delete the AWS Glue crawler.
    required: true
    choices: [ 'present', 'absent' ]
    type: str
  table_prefix:
    description:
      - The table prefix used for catalog tables that are created.
    type: str
  targets:
    description:
      - A list of targets to crawl. See example below.
      - Required when I(state=present).
    type: dict
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Create an AWS Glue crawler
- community.aws.glue_crawler:
    name: my-glue-crawler
    database_name: my_database
    role: my-iam-role
    schema_change_policy:
      delete_behavior: DELETE_FROM_DATABASE
      update_behavior: UPDATE_IN_DATABASE
    recrawl_policy:
      recrawl_ehavior: CRAWL_EVERYTHING
    targets:
      S3Targets:
        - Path: "s3://my-bucket/prefix/folder/"
          ConnectionName: my-connection
          Exclusions:
            - "**.json"
            - "**.yml"
    state: present

# Delete an AWS Glue crawler
- community.aws.glue_crawler:
    name: my-glue-crawler
    state: absent
"""

RETURN = r"""
creation_time:
    description: The time and date that this crawler definition was created.
    returned: when state is present
    type: str
    sample: '2021-04-01T05:19:58.326000+00:00'
database_name:
    description: The name of the database where results are written.
    returned: when state is present
    type: str
    sample: my_table
description:
    description: Description of the crawler.
    returned: when state is present
    type: str
    sample: My crawler
last_updated:
    description: The time and date that this crawler definition was last updated.
    returned: when state is present
    type: str
    sample: '2021-04-01T05:19:58.326000+00:00'
name:
    description: The name of the AWS Glue crawler.
    returned: always
    type: str
    sample: my-glue-crawler
recrawl_policy:
    description: A policy that specifies whether to crawl the entire dataset again, or to crawl only folders that were added since the last crawler run.
    returned: when state is present
    type: complex
    contains:
        RecrawlBehavior:
            description: Whether to crawl the entire dataset again or to crawl only folders that were added since the last crawler run.
            returned: when state is present
            type: str
            sample: CRAWL_EVERYTHING
role:
    description: The name or ARN of the IAM role associated with this crawler.
    returned: when state is present
    type: str
    sample: my-iam-role
schema_change_policy:
    description: The policy for the crawler's update and deletion behavior.
    returned: when state is present
    type: complex
    contains:
        DeleteBehavior:
            description: The deletion behavior when the crawler finds a deleted object.
            returned: when state is present
            type: str
            sample: DELETE_FROM_DATABASE
        UpdateBehavior:
            description: The update behavior when the crawler finds a changed schema.
            returned: when state is present
            type: str
            sample: UPDATE_IN_DATABASE

table_prefix:
    description: The table prefix used for catalog tables that are created.
    returned: when state is present
    type: str
    sample: my_prefix
targets:
    description: A list of targets to crawl.
    returned: when state is present
    type: complex
    contains:
        S3Targets:
            description: List of S3 targets.
            returned: when state is present
            type: list
        JdbcTargets:
            description: List of JDBC targets.
            returned: when state is present
            type: list
        MongoDBTargets:
            description: List of Mongo DB targets.
            returned: when state is present
            type: list
        DynamoDBTargets:
            description: List of DynamoDB targets.
            returned: when state is present
            type: list
        CatalogTargets:
            description: List of catalog targets.
            returned: when state is present
            type: list
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.iam import get_aws_account_info
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def _get_glue_crawler(connection, module, glue_crawler_name):
    """
    Get an AWS Glue crawler based on name. If not found, return None.
    """
    try:
        return connection.get_crawler(aws_retry=True, Name=glue_crawler_name)["Crawler"]
    except is_boto3_error_code("EntityNotFoundException"):
        return None
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e)


def _trim_targets(targets):
    return [_trim_target(t) for t in targets]


def _trim_target(target):
    """
    Some target types have optional parameters which AWS will fill in and return
    To compare the desired targets and the current targets we need to ignore the defaults
    """
    if not target:
        return None
    retval = target.copy()
    if not retval.get("Exclusions", None):
        retval.pop("Exclusions", None)
    return retval


def _compare_glue_crawler_params(user_params, current_params):
    """
    Compare Glue crawler params. If there is a difference, return True immediately else return False
    """
    if "DatabaseName" in user_params and user_params["DatabaseName"] != current_params["DatabaseName"]:
        return True
    if "Description" in user_params and user_params["Description"] != current_params["Description"]:
        return True
    if "RecrawlPolicy" in user_params and user_params["RecrawlPolicy"] != current_params["RecrawlPolicy"]:
        return True
    if "Role" in user_params and user_params["Role"] != current_params["Role"]:
        return True
    if (
        "SchemaChangePolicy" in user_params
        and user_params["SchemaChangePolicy"] != current_params["SchemaChangePolicy"]
    ):
        return True
    if "TablePrefix" in user_params and user_params["TablePrefix"] != current_params["TablePrefix"]:
        return True
    if "Targets" in user_params:
        if "S3Targets" in user_params["Targets"]:
            if _trim_targets(user_params["Targets"]["S3Targets"]) != _trim_targets(
                current_params["Targets"]["S3Targets"]
            ):
                return True
        if (
            "JdbcTargets" in user_params["Targets"]
            and user_params["Targets"]["JdbcTargets"] != current_params["Targets"]["JdbcTargets"]
        ):
            if _trim_targets(user_params["Targets"]["JdbcTargets"]) != _trim_targets(
                current_params["Targets"]["JdbcTargets"]
            ):
                return True
        if (
            "MongoDBTargets" in user_params["Targets"]
            and user_params["Targets"]["MongoDBTargets"] != current_params["Targets"]["MongoDBTargets"]
        ):
            return True
        if (
            "DynamoDBTargets" in user_params["Targets"]
            and user_params["Targets"]["DynamoDBTargets"] != current_params["Targets"]["DynamoDBTargets"]
        ):
            return True
        if (
            "CatalogTargets" in user_params["Targets"]
            and user_params["Targets"]["CatalogTargets"] != current_params["Targets"]["CatalogTargets"]
        ):
            return True

    return False


def ensure_tags(connection, module, glue_crawler):
    changed = False

    if module.params.get("tags") is None:
        return False

    account_id, partition = get_aws_account_info(module)
    arn = f"arn:{partition}:glue:{module.region}:{account_id}:crawler/{module.params.get('name')}"

    try:
        existing_tags = connection.get_tags(aws_retry=True, ResourceArn=arn).get("Tags", {})
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        if module.check_mode:
            existing_tags = {}
        else:
            module.fail_json_aws(e, msg=f"Unable to get tags for Glue crawler {module.params.get('name')}")

    tags_to_add, tags_to_remove = compare_aws_tags(
        existing_tags, module.params.get("tags"), module.params.get("purge_tags")
    )

    if tags_to_remove:
        changed = True
        if not module.check_mode:
            try:
                connection.untag_resource(aws_retry=True, ResourceArn=arn, TagsToRemove=tags_to_remove)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set tags for Glue crawler {module.params.get('name')}")

    if tags_to_add:
        changed = True
        if not module.check_mode:
            try:
                connection.tag_resource(aws_retry=True, ResourceArn=arn, TagsToAdd=tags_to_add)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg=f"Unable to set tags for Glue crawler {module.params.get('name')}")

    return changed


def create_or_update_glue_crawler(connection, module, glue_crawler):
    """
    Create or update an AWS Glue crawler
    """

    changed = False
    params = dict()
    params["Name"] = module.params.get("name")
    params["Role"] = module.params.get("role")
    params["Targets"] = module.params.get("targets")
    if module.params.get("database_name") is not None:
        params["DatabaseName"] = module.params.get("database_name")
    if module.params.get("description") is not None:
        params["Description"] = module.params.get("description")
    if module.params.get("recrawl_policy") is not None:
        params["RecrawlPolicy"] = snake_dict_to_camel_dict(module.params.get("recrawl_policy"), capitalize_first=True)
    if module.params.get("role") is not None:
        params["Role"] = module.params.get("role")
    if module.params.get("schema_change_policy") is not None:
        params["SchemaChangePolicy"] = snake_dict_to_camel_dict(
            module.params.get("schema_change_policy"), capitalize_first=True
        )
    if module.params.get("table_prefix") is not None:
        params["TablePrefix"] = module.params.get("table_prefix")
    if module.params.get("targets") is not None:
        params["Targets"] = module.params.get("targets")

    if glue_crawler:
        if _compare_glue_crawler_params(params, glue_crawler):
            try:
                if not module.check_mode:
                    connection.update_crawler(aws_retry=True, **params)
                changed = True
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e)
    else:
        try:
            if not module.check_mode:
                connection.create_crawler(aws_retry=True, **params)
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e)

    glue_crawler = _get_glue_crawler(connection, module, params["Name"])

    changed |= ensure_tags(connection, module, glue_crawler)

    module.exit_json(
        changed=changed,
        **camel_dict_to_snake_dict(glue_crawler or {}, ignore_list=["SchemaChangePolicy", "RecrawlPolicy", "Targets"]),
    )


def delete_glue_crawler(connection, module, glue_crawler):
    """
    Delete an AWS Glue crawler
    """
    changed = False

    if glue_crawler:
        try:
            if not module.check_mode:
                connection.delete_crawler(aws_retry=True, Name=glue_crawler["Name"])
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e)

    module.exit_json(changed=changed)


def main():
    argument_spec = dict(
        database_name=dict(type="str"),
        description=dict(type="str"),
        name=dict(required=True, type="str"),
        purge_tags=dict(type="bool", default=True),
        recrawl_policy=dict(type="dict", options=dict(recrawl_behavior=dict(type="str"))),
        role=dict(type="str"),
        schema_change_policy=dict(
            type="dict", options=dict(delete_behavior=dict(type="str"), update_behavior=dict(type="str"))
        ),
        state=dict(required=True, choices=["present", "absent"], type="str"),
        table_prefix=dict(type="str"),
        tags=dict(type="dict", aliases=["resource_tags"]),
        targets=dict(type="dict"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[("state", "present", ["role", "targets"])],
        supports_check_mode=True,
    )

    connection = module.client("glue", retry_decorator=AWSRetry.jittered_backoff(retries=10))

    state = module.params.get("state")

    glue_crawler = _get_glue_crawler(connection, module, module.params.get("name"))

    if state == "present":
        create_or_update_glue_crawler(connection, module, glue_crawler)
    else:
        delete_glue_crawler(connection, module, glue_crawler)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template
short_description: Manage templates in schemas
description:
- Manage templates on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
options:
  tenant:
    description:
    - The tenant used for this template.
    type: str
    required: true
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  schema_description:
    description:
    - The description of Schema is supported on versions of MSO that are 3.3 or greater.
    type: str
  template_description:
    description:
    - The description of template is supported on versions of MSO that are 3.3 or greater.
    type: str
  template:
    description:
    - The name of the template.
    type: str
    aliases: [ name ]
  display_name:
    description:
    - The name as displayed on the MSO web interface.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
notes:
- Due to restrictions of the MSO REST API this module creates schemas when needed, and removes them when the last template has been removed.
seealso:
- module: cisco.mso.mso_schema
- module: cisco.mso.mso_schema_site
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new template to a schema
  cisco.mso.mso_schema_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: Tenant 1
    schema: Schema 1
    template: Template 1
    state: present
  delegate_to: localhost

- name: Remove a template from a schema
  cisco.mso.mso_schema_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: Tenant 1
    schema: Schema 1
    template: Template 1
    state: absent
  delegate_to: localhost

- name: Query a template
  cisco.mso.mso_schema_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: Tenant 1
    schema: Schema 1
    template: Template 1
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all templates
  cisco.mso.mso_schema_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: Tenant 1
    schema: Schema 1
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        tenant=dict(type="str", required=True),
        schema=dict(type="str", required=True),
        schema_description=dict(type="str"),
        template_description=dict(type="str"),
        template=dict(type="str", aliases=["name"]),
        display_name=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["template"]],
            ["state", "present", ["template"]],
        ],
    )

    tenant = module.params.get("tenant")
    schema = module.params.get("schema")
    schema_description = module.params.get("schema_description")
    template_description = module.params.get("template_description")
    template = module.params.get("template")
    if template is not None:
        template = template.replace(" ", "")
    display_name = module.params.get("display_name")
    state = module.params.get("state")

    mso = MSOModule(module)

    # Get schema
    schema_obj = mso.get_obj("schemas", displayName=schema)

    mso.existing = {}
    if schema_obj:
        # Schema exists
        schema_path = "schemas/{id}".format(**schema_obj)

        # Get template
        templates = [t.get("name") for t in schema_obj.get("templates")]
        if template:
            if template in templates:
                template_idx = templates.index(template)
                mso.existing = schema_obj.get("templates")[template_idx]
        else:
            mso.existing = schema_obj.get("templates")
    else:
        schema_path = "schemas"

    if state == "query":
        if not mso.existing:
            if template:
                mso.fail_json(msg="Template '{0}' not found".format(template))
            else:
                mso.existing = []
        mso.exit_json()

    template_path = "/templates/{0}".format(template)
    ops = []

    mso.previous = mso.existing
    if state == "absent":
        mso.proposed = mso.sent = {}
        if not schema_obj:
            # There was no schema to begin with
            pass
        elif len(templates) == 1 and mso.existing:
            # There is only one tenant, remove schema
            mso.existing = {}
            if not module.check_mode:
                mso.request(schema_path, method="DELETE")
        elif mso.existing:
            # Remove existing template
            mso.existing = {}
            ops.append(dict(op="remove", path=template_path))

    elif state == "present":
        tenant_id = mso.lookup_tenant(tenant)

        if display_name is None:
            display_name = mso.existing.get("displayName", template)

        if not schema_obj:
            # Schema does not exist, so we have to create it
            payload = dict(
                displayName=schema,
                templates=[
                    dict(
                        name=template,
                        displayName=display_name,
                        tenantId=tenant_id,
                    )
                ],
                sites=[],
            )

            if schema_description is not None:
                payload.update(description=schema_description)
            if template_description is not None:
                payload["templates"][0].update(description=template_description)

            mso.existing = payload.get("templates")[0]

            if not module.check_mode:
                mso.request(schema_path, method="POST", data=payload)

        elif mso.existing:
            # Template exists, so we have to update it
            payload = dict(
                name=template,
                displayName=display_name,
                description=template_description,
                tenantId=tenant_id,
            )

            mso.sanitize(payload, collate=True)

            ops.append(dict(op="replace", path=template_path + "/displayName", value=display_name))
            ops.append(dict(op="replace", path=template_path + "/tenantId", value=tenant_id))

            mso.existing = mso.proposed
        else:
            # Template does not exist, so we have to add it
            payload = dict(
                name=template,
                displayName=display_name,
                tenantId=tenant_id,
            )

            mso.sanitize(payload, collate=True)

            ops.append(dict(op="add", path="/templates/-", value=payload))

            mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(schema_path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Michael DeHaan <michael.dehaan@gmail.com>, 2012-2013
#
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import hmac
import re

from ansible.module_utils.six.moves.urllib.parse import urlparse

try:
    from hashlib import sha1
except ImportError:
    import sha as sha1

HASHED_KEY_MAGIC = "|1|"


def is_ssh_url(url):

    """ check if url is ssh """

    if "@" in url and "://" not in url:
        return True
    for scheme in "ssh://", "git+ssh://", "ssh+git://":
        if url.startswith(scheme):
            return True
    return False


def get_fqdn_and_port(repo_url):

    """ chop the hostname and port out of a url """

    fqdn = None
    port = None
    ipv6_re = re.compile(r'(\[[^]]*\])(?::([0-9]+))?')
    if "@" in repo_url and "://" not in repo_url:
        # most likely an user@host:path or user@host/path type URL
        repo_url = repo_url.split("@", 1)[1]
        match = ipv6_re.match(repo_url)
        # For this type of URL, colon specifies the path, not the port
        if match:
            fqdn, path = match.groups()
        elif ":" in repo_url:
            fqdn = repo_url.split(":")[0]
        elif "/" in repo_url:
            fqdn = repo_url.split("/")[0]
    elif "://" in repo_url:
        # this should be something we can parse with urlparse
        parts = urlparse(repo_url)
        # parts[1] will be empty on python2.4 on ssh:// or git:// urls, so
        # ensure we actually have a parts[1] before continuing.
        if parts[1] != '':
            fqdn = parts[1]
            if "@" in fqdn:
                fqdn = fqdn.split("@", 1)[1]
            match = ipv6_re.match(fqdn)
            if match:
                fqdn, port = match.groups()
            elif ":" in fqdn:
                fqdn, port = fqdn.split(":")[0:2]
    return fqdn, port


def check_hostkey(module, fqdn):
    return not not_in_host_file(module, fqdn)


# this is a variant of code found in connection_plugins/paramiko.py and we should modify
# the paramiko code to import and use this.

def not_in_host_file(self, host):

    if 'USER' in os.environ:
        user_host_file = os.path.expandvars("~${USER}/.ssh/known_hosts")
    else:
        user_host_file = "~/.ssh/known_hosts"
    user_host_file = os.path.expanduser(user_host_file)

    host_file_list = [
        user_host_file,
        "/etc/ssh/ssh_known_hosts",
        "/etc/ssh/ssh_known_hosts2",
        "/etc/openssh/ssh_known_hosts",
    ]

    hfiles_not_found = 0
    for hf in host_file_list:
        if not os.path.exists(hf):
            hfiles_not_found += 1
            continue

        try:
            host_fh = open(hf)
        except IOError:
            hfiles_not_found += 1
            continue
        else:
            data = host_fh.read()
            host_fh.close()

        for line in data.split("\n"):
            if line is None or " " not in line:
                continue
            tokens = line.split()
            if tokens[0].find(HASHED_KEY_MAGIC) == 0:
                # this is a hashed known host entry
                try:
                    (kn_salt, kn_host) = tokens[0][len(HASHED_KEY_MAGIC):].split("|", 2)
                    hash = hmac.new(kn_salt.decode('base64'), digestmod=sha1)
                    hash.update(host)
                    if hash.digest() == kn_host.decode('base64'):
                        return False
                except Exception:
                    # invalid hashed host key, skip it
                    continue
            else:
                # standard host file entry
                if host in tokens[0]:
                    return False

    return True


def add_host_key(module, fqdn, port=22, key_type="rsa", create_dir=False):

    """ use ssh-keyscan to add the hostkey """

    keyscan_cmd = module.get_bin_path('ssh-keyscan', True)

    if 'USER' in os.environ:
        user_ssh_dir = os.path.expandvars("~${USER}/.ssh/")
        user_host_file = os.path.expandvars("~${USER}/.ssh/known_hosts")
    else:
        user_ssh_dir = "~/.ssh/"
        user_host_file = "~/.ssh/known_hosts"
    user_ssh_dir = os.path.expanduser(user_ssh_dir)

    if not os.path.exists(user_ssh_dir):
        if create_dir:
            try:
                os.makedirs(user_ssh_dir, int('700', 8))
            except Exception:
                module.fail_json(msg="failed to create host key directory: %s" % user_ssh_dir)
        else:
            module.fail_json(msg="%s does not exist" % user_ssh_dir)
    elif not os.path.isdir(user_ssh_dir):
        module.fail_json(msg="%s is not a directory" % user_ssh_dir)

    if port:
        this_cmd = "%s -t %s -p %s %s" % (keyscan_cmd, key_type, port, fqdn)
    else:
        this_cmd = "%s -t %s %s" % (keyscan_cmd, key_type, fqdn)

    rc, out, err = module.run_command(this_cmd)
    # ssh-keyscan gives a 0 exit code and prints nothing on timeout
    if rc != 0 or not out:
        msg = 'failed to retrieve hostkey'
        if not out:
            msg += '. "%s" returned no matches.' % this_cmd
        else:
            msg += ' using command "%s". [stdout]: %s' % (this_cmd, out)

        if err:
            msg += ' [stderr]: %s' % err

        module.fail_json(msg=msg)

    module.append_to_file(user_host_file, out)

    return rc, out, err



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021, Ciril Troxler <ciril.troxler@cloudscale.ch>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: custom_image
short_description: Manage custom images on the cloudscale.ch IaaS service
description:
  - Import, modify and delete custom images.
notes:
  - To import a new custom-image the I(url) and I(name) options are required.
author:
  - Ciril Troxler (@ctx)
  - Gaudenz Steinlin (@gaudenz)
version_added: 2.2.0
options:
  url:
    description:
      - The URL used to download the image.
    type: str
  force_retry:
    description:
      - Retry the image import even if a failed import using the same name and
        URL already exists. This is necessary to recover from download errors.
    default: false
    type: bool
  name:
    description:
      - The human readable name of the custom image. Either name or UUID must
        be present to change an existing image.
    type: str
  uuid:
    description:
      - The unique identifier of the custom image import. Either name or UUID
        must be present to change an existing image.
    type: str
  slug:
    description:
      - A string identifying the custom image for use within the API.
    type: str
  user_data_handling:
    description:
      - How user_data will be handled when creating a server. There are
        currently two options, "pass-through" and "extend-cloud-config".
    type: str
    choices: [ pass-through, extend-cloud-config ]
  zones:
    description:
      - Specify zones in which the custom image will be available (e.g. C(lpg1)
        or C(rma1)).
    type: list
    elements: str
  source_format:
    description:
      - The file format of the image referenced in the url. Currently only raw
        is supported.
    type: str
  firmware_type:
    description:
      - The firmware type that will be used for servers created
        with this image.
    type: str
    choices: [ bios, uefi ]
    default: bios
  tags:
    description:
      - The tags assigned to the custom image.
    type: dict
  state:
    description: State of the coustom image.
    choices: [ present, absent ]
    default: present
    type: str
extends_documentation_fragment: cloudscale_ch.cloud.api_parameters
'''

EXAMPLES = r'''
- name: Import custom image
  cloudscale_ch.cloud.custom_image:
    name: "My Custom Image"
    url: https://ubuntu.com/downloads/hirsute.img
    slug: my-custom-image
    user_data_handling: extend-cloud-config
    zones: lpg1
    tags:
      project: luna
    state: present
  register: my_custom_image

- name: Wait until import succeeded
  cloudscale_ch.cloud.custom_image:
    uuid: "{{ my_custom_image.uuid }}"
  retries: 15
  delay: 5
  register: image
  until: image.import_status == 'success'
  failed_when: image.import_status == 'failed'

- name: Import custom image and wait until import succeeded
  cloudscale_ch.cloud.custom_image:
    name: "My Custom Image"
    url: https://ubuntu.com/downloads/hirsute.img
    slug: my-custom-image
    user_data_handling: extend-cloud-config
    zones: lpg1
    tags:
      project: luna
    state: present
  retries: 15
  delay: 5
  register: image
  until: image.import_status == 'success'
  failed_when: image.import_status == 'failed'

- name: Import custom image with UEFI firmware type
  cloudscale_ch.cloud.custom_image:
    name: "My Custom UEFI Image"
    url: https://ubuntu.com/downloads/hirsute.img
    slug: my-custom-uefi-image
    user_data_handling: extend-cloud-config
    zones: lpg1
    firmware_type: uefi
    tags:
      project: luna
    state: present
  register: my_custom_image

- name: Update custom image
  cloudscale_ch.cloud.custom_image:
    name: "My Custom Image"
    slug: my-custom-image
    user_data_handling: extend-cloud-config
    tags:
      project: luna
    state: present

- name: Delete custom image
  cloudscale_ch.cloud.custom_image:
    uuid: '{{ my_custom_image.uuid }}'
    state: absent

- name: List all custom images
  uri:
    url: 'https://api.cloudscale.ch/v1/custom-images'
    headers:
      Authorization: 'Bearer {{ query("env", "CLOUDSCALE_API_TOKEN") }}'
    status_code: 200
  register: image_list
- name: Search the image list for all images with name 'My Custom Image'
  set_fact:
    my_custom_images: '{{ image_list.json | selectattr("name","search", "My Custom Image" ) }}'
'''

RETURN = r'''
href:
  description: The API URL to get details about this resource.
  returned: success when state == present
  type: str
  sample: https://api.cloudscale.ch/v1/custom-imges/11111111-1864-4608-853a-0771b6885a3a
uuid:
  description: The unique identifier of the custom image.
  returned: success
  type: str
  sample: 11111111-1864-4608-853a-0771b6885a3a
name:
  description: The human readable name of the custom image.
  returned: success
  type: str
  sample: alan
created_at:
  description: The creation date and time of the resource.
  returned: success
  type: str
  sample: "2020-05-29T13:18:42.511407Z"
slug:
  description: A string identifying the custom image for use within the API.
  returned: success
  type: str
  sample: foo
checksums:
  description: The checksums of the custom image as key and value pairs. The
    algorithm (e.g. sha256) name is in the key and the checksum in the value.
    The set of algorithms used might change in the future.
  returned: success
  type: dict
  sample: {
    "md5": "5b3a1f21cde154cfb522b582f44f1a87",
    "sha256": "5b03bcbd00b687e08791694e47d235a487c294e58ca3b1af704120123aa3f4e6"
  }
user_data_handling:
  description: How user_data will be handled when creating a server. There are
    currently two options, "pass-through" and "extend-cloud-config".
  returned: success
  type: str
  sample: "pass-through"
tags:
  description: Tags assosiated with the custom image.
  returned: success
  type: dict
  sample: { 'project': 'my project' }
import_status:
  description: Shows the progress of an import. Values are one of
    "started", "in_progress", "success" or "failed".
  returned: success
  type: str
  sample: "in_progress"
error_message:
  description: Error message in case of a failed import.
  returned: success
  type: str
  sample: "Expected HTTP 200, got HTTP 403"
state:
  description: The current status of the custom image.
  returned: success
  type: str
  sample: present
'''


from ansible.module_utils.basic import (
    AnsibleModule,
)
from ansible.module_utils.urls import (
    fetch_url
)
from ..module_utils.api import (
    AnsibleCloudscaleBase,
    cloudscale_argument_spec,
)
from ansible.module_utils._text import (
    to_text
)


class AnsibleCloudscaleCustomImage(AnsibleCloudscaleBase):

    def _transform_import_to_image(self, imp):
        # Create a stub image from the import
        img = imp.get('custom_image', {})
        return {
            'href': img.get('href'),
            'uuid': imp['uuid'],
            'name': img.get('name'),
            'created_at': None,
            'size_gb': None,
            'checksums': None,
            'tags': imp['tags'],
            'url': imp['url'],
            'import_status': imp['status'],
            'error_message': imp.get('error_message', ''),
            # Even failed image imports are reported as present. This then
            # represents a failed import resource.
            'state': 'present',
            # These fields are not present on the import, assume they are
            # unchanged from the module parameters
            'user_data_handling': self._module.params['user_data_handling'],
            'zones': self._module.params['zones'],
            'slug': self._module.params['slug'],
            'firmware_type': self._module.params['firmware_type'],
        }

    # This method can be replaced by calling AnsibleCloudscaleBase._get form
    # AnsibleCloudscaleCustomImage._get once the API bug is fixed.
    def _get_url(self, url):

        response, info = fetch_url(self._module,
                                   url,
                                   headers=self._auth_header,
                                   method='GET',
                                   timeout=self._module.params['api_timeout'])

        if info['status'] == 200:
            response = self._module.from_json(
                to_text(response.read(),
                        errors='surrogate_or_strict'),
            )
        elif info['status'] == 404:
            # Return None to be compatible with AnsibleCloudscaleBase._get
            response = None
        elif info['status'] == 500 and url.startswith(self._api_url + self.resource_name + '/import/'):
            # Workaround a bug in the cloudscale.ch API which wrongly returns
            # 500 instead of 404
            response = None
        else:
            self._module.fail_json(
                msg='Failure while calling the cloudscale.ch API with GET for '
                '"%s"' % url,
                fetch_url_info=info,
            )

        return response

    def _get(self, api_call):

        # Split api_call into components
        api_url, call_uuid = api_call.split(self.resource_name)

        # If the api_call does not contain the API URL
        if not api_url:
            api_url = self._api_url

        # Fetch image(s) from the regular API endpoint
        response = self._get_url(api_url + self.resource_name + call_uuid) or []

        # Additionally fetch image(s) from the image import API endpoint
        response_import = self._get_url(
            api_url + self.resource_name + '/import' + call_uuid,
        ) or []

        # No image was found
        if call_uuid and response == [] and response_import == []:
            return None

        # Convert single image responses (call with UUID) into a list
        if call_uuid and response:
            response = [response]
        if call_uuid and response_import:
            response_import = [response_import]

        # Transform lists into UUID keyed dicts
        response = dict([(i['uuid'], i) for i in response])
        response_import = dict([(i['uuid'], i) for i in response_import])

        # Filter the import list so that successfull and in_progress imports
        # shadow failed imports
        response_import_filtered = dict([(k, v) for k, v
                                         in response_import.items()
                                         if v['status'] in ('success',
                                                            'in_progress')])
        # Only add failed imports if no import with the same name exists
        # Only add the last failed import in the list (there is no timestamp on
        # imports)
        import_names = set([v['custom_image']['name'] for k, v
                           in response_import_filtered.items()])
        for k, v in reversed(list(response_import.items())):
            name = v['custom_image']['name']
            if (v['status'] == 'failed' and name not in import_names):
                import_names.add(name)
                response_import_filtered[k] = v

        # Merge import list into image list
        for uuid, imp in response_import_filtered.items():
            if uuid in response:
                # Merge addtional fields only present on the import
                response[uuid].update(
                    url=imp['url'],
                    import_status=imp['status'],
                    error_message=imp.get('error_message', ''),
                )
            else:
                response[uuid] = self._transform_import_to_image(imp)

        if not call_uuid:
            return response.values()
        else:
            return next(iter(response.values()))

    def _post(self, api_call, data=None):
        # Only new image imports are supported, no direct POST call to image
        # resources are supported by the API
        if not api_call.endswith('custom-images'):
            self._module.fail_json(msg="Error: Bad api_call URL.")
        # Custom image imports use a different endpoint
        api_call += '/import'

        if self._module.params['url']:
            return self._transform_import_to_image(
                self._post_or_patch("%s" % api_call, 'POST', data),
            )
        else:
            self._module.fail_json(msg="Cannot import a new image without url.")

    def present(self):
        resource = self.query()

        # If the module passes the firmware_type argument,
        # and the module argument and API response are not the same for
        # argument firmware_type.
        if (resource.get('firmware_type') is not None
                and resource.get('firmware_type') !=
                self._module.params['firmware_type']):
            # Custom error if the module tries to change the firmware_type.
            msg = "Cannot change firmware type of an existing custom image"
            self._module.fail_json(msg)

        if resource['state'] == "absent":
            resource = self.create(resource)
        else:
            # If this is a failed upload and the URL changed or the "force_retry"
            # parameter is used, create a new image import.
            if (resource.get('import_status') == 'failed'
                and (resource['url'] != self._module.params['url']
                     or self._module.params['force_retry'])):
                resource = self.create(resource)
            else:
                resource = self.update(resource)

        return self.get_result(resource)


def main():
    argument_spec = cloudscale_argument_spec()
    argument_spec.update(dict(
        name=dict(type='str'),
        slug=dict(type='str'),
        url=dict(type='str'),
        force_retry=dict(type='bool', default=False),
        user_data_handling=dict(type='str',
                                choices=('pass-through',
                                         'extend-cloud-config')),
        uuid=dict(type='str'),
        firmware_type=dict(type='str',
                           choices=('bios',
                                    'uefi'),
                           default=('bios')),
        tags=dict(type='dict'),
        state=dict(type='str', default='present',
                   choices=('present', 'absent')),
        zones=dict(type='list', elements='str'),
        source_format=dict(type='str'),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=(('name', 'uuid'),),
        supports_check_mode=True,
    )

    cloudscale_custom_image = AnsibleCloudscaleCustomImage(
        module,
        resource_name='custom-images',
        resource_key_uuid='uuid',
        resource_key_name='name',
        resource_create_param_keys=[
            'name',
            'slug',
            'url',
            'user_data_handling',
            'firmware_type',
            'tags',
            'zones',
            'source_format',
        ],
        resource_update_param_keys=[
            'name',
            'slug',
            'user_data_handling',
            'firmware_type',
            'tags',
        ],
    )

    if module.params['state'] == "absent":
        result = cloudscale_custom_image.absent()
    else:
        result = cloudscale_custom_image.present()
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: licensing_tier_state_create
short_description: Resource module for Licensing Tier State Create
description:
- Manage operation create of the resource Licensing Tier State Create.
- Applicable values for **name** & **status** parameters.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Licensing Tier State Create's payload.
    elements: dict
    suboptions:
      name:
        description: Licensing Tier State Create's name.
        type: str
      status:
        description: Licensing Tier State Create's status.
        type: str
    type: list
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Licensing
  description: Complete reference of the Licensing API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!license-openapi
notes:
  - SDK Method used are
    licensing.Licensing.update_tier_state_info,

  - Paths used are
    post /api/v1/license/system/tier-state,

"""

EXAMPLES = r"""
- name: Update all
  cisco.ise.licensing_tier_state_create:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    payload:
    - name: string
      status: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "message": "string",
          "name": "string",
          "status": "string"
        }
      ],
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The eos logging_global fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.logging_global.logging_global import (
    Logging_globalArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.rm_templates.logging_global import (
    Logging_globalTemplate,
)


class Logging_globalFacts(object):
    """The eos logging_global facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Logging_globalArgs.argument_spec

    def get_config(self, connection):
        """Wrapper method for `connection.get()`
        This method exists solely to allow the unit test framework to mock device connection calls.
        """
        return connection.get("show running-config | section logging")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Logging_global network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = {}

        if not data:
            data = self.get_config(connection)

        # parse native config using the Prefix_lists template
        logging_parser = Logging_globalTemplate(
            lines=data.splitlines(),
            module=self._module,
        )
        objs = logging_parser.parse()
        if objs:
            if "hosts" in objs:
                objs["hosts"] = sorted(
                    list(objs["hosts"].values()),
                    key=lambda k, sk="name": k[sk],
                )
            if "vrfs" in objs:
                for k, v in iteritems(objs["vrfs"]):
                    if "hosts" in v:
                        v["hosts"] = sorted(
                            list(v["hosts"].values()),
                            key=lambda k, sk="name": k[sk],
                        )
                objs["vrfs"] = sorted(
                    list(objs["vrfs"].values()),
                    key=lambda k, sk="name": k[sk],
                )
        else:
            objs = {}
        ansible_facts["ansible_network_resources"].pop("logging_global", None)

        params = utils.remove_empties(
            logging_parser.validate_config(
                self.argument_spec,
                {"config": objs},
                redact=True,
            ),
        )

        facts["logging_global"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_rf_profile
short_description: Resource module for Wireless Rf Profile
description:
- Manage operations create and delete of the resource Wireless Rf Profile.
- Create or Update RF profile.
- Delete RF profiles.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  channelWidth:
    description: Channel Width.
    type: str
  defaultRfProfile:
    description: Is Default Rf Profile.
    type: bool
  enableBrownField:
    description: Enable Brown Field.
    type: bool
  enableCustom:
    description: Enable Custom.
    type: bool
  enableRadioTypeA:
    description: Enable Radio Type A.
    type: bool
  enableRadioTypeB:
    description: Enable Radio Type B.
    type: bool
  enableRadioTypeC:
    description: Enable Radio Type C (6GHz).
    type: bool
  name:
    description: RF Profile Name.
    type: str
  radioTypeAProperties:
    description: Wireless Rf Profile's radioTypeAProperties.
    suboptions:
      dataRates:
        description: Data Rates.
        type: str
      mandatoryDataRates:
        description: Mandatory Data Rates.
        type: str
      maxPowerLevel:
        description: Max Power Level.
        type: int
      minPowerLevel:
        description: Rx Sop Threshold.
        type: int
      parentProfile:
        description: Parent Profile.
        type: str
      powerThresholdV1:
        description: Power Threshold V1.
        type: int
      radioChannels:
        description: Radio Channels.
        type: str
      rxSopThreshold:
        description: Rx Sop Threshold.
        type: str
    type: dict
  radioTypeBProperties:
    description: Wireless Rf Profile's radioTypeBProperties.
    suboptions:
      dataRates:
        description: Data Rates.
        type: str
      mandatoryDataRates:
        description: Mandatory Data Rates.
        type: str
      maxPowerLevel:
        description: Max Power Level.
        type: int
      minPowerLevel:
        description: Min Power Level.
        type: int
      parentProfile:
        description: Parent Profile.
        type: str
      powerThresholdV1:
        description: Power Threshold V1.
        type: int
      radioChannels:
        description: Radio Channels.
        type: str
      rxSopThreshold:
        description: Rx Sop Threshold.
        type: str
    type: dict
  radioTypeCProperties:
    description: Wireless Rf Profile's radioTypeCProperties.
    suboptions:
      dataRates:
        description: Data Rates.
        type: str
      mandatoryDataRates:
        description: Mandatory Data Rates.
        type: str
      maxPowerLevel:
        description: Max Power Level.
        type: int
      minPowerLevel:
        description: Min Power Level.
        type: int
      parentProfile:
        description: Parent Profile.
        type: str
      powerThresholdV1:
        description: Power Threshold V1.
        type: int
      radioChannels:
        description: Radio Channels.
        type: str
      rxSopThreshold:
        description: Rx Sop Threshold.
        type: str
    type: dict
  rfProfileName:
    description: RfProfileName path parameter. RF profile name to be deleted(required)
      *non-custom RF profile cannot be deleted.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless CreateOrUpdateRFProfile
  description: Complete reference of the CreateOrUpdateRFProfile API.
  link: https://developer.cisco.com/docs/dna-center/#!create-or-update-rf-profile
- name: Cisco DNA Center documentation for Wireless DeleteRFProfiles
  description: Complete reference of the DeleteRFProfiles API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-rf-profiles
notes:
  - SDK Method used are
    wireless.Wireless.create_or_update_rf_profile,
    wireless.Wireless.delete_rf_profiles,

  - Paths used are
    post /dna/intent/api/v1/wireless/rf-profile,
    delete /dna/intent/api/v1/wireless/rf-profile/{rfProfileName},

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.wireless_rf_profile:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    channelWidth: string
    defaultRfProfile: true
    enableBrownField: true
    enableCustom: true
    enableRadioTypeA: true
    enableRadioTypeB: true
    enableRadioTypeC: true
    name: string
    radioTypeAProperties:
      dataRates: string
      mandatoryDataRates: string
      maxPowerLevel: 0
      minPowerLevel: 0
      parentProfile: string
      powerThresholdV1: 0
      radioChannels: string
      rxSopThreshold: string
    radioTypeBProperties:
      dataRates: string
      mandatoryDataRates: string
      maxPowerLevel: 0
      minPowerLevel: 0
      parentProfile: string
      powerThresholdV1: 0
      radioChannels: string
      rxSopThreshold: string
    radioTypeCProperties:
      dataRates: string
      mandatoryDataRates: string
      maxPowerLevel: 0
      minPowerLevel: 0
      parentProfile: string
      powerThresholdV1: 0
      radioChannels: string
      rxSopThreshold: string

- name: Delete by name
  cisco.dnac.wireless_rf_profile:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    rfProfileName: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "executionId": "string",
      "executionUrl": "string",
      "message": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: Ansible Project
# Copyright: (c) 2021, Mark Mercado <mmercado@digitalocean.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: digital_ocean_database_info
short_description: Gather information about DigitalOcean databases
description:
  - Gather information about DigitalOcean databases.
version_added: 1.3.0
author: "Mark Mercado (@mamercad)"
options:
  id:
    description:
      - A unique ID that can be used to identify and reference a database cluster.
    type: int
    aliases: ['database_id']
    required: false
  name:
    description:
      - A unique, human-readable name for the database cluster.
    type: str
    required: false
extends_documentation_fragment:
  - community.digitalocean.digital_ocean.documentation
"""


EXAMPLES = r"""
- name: Gather all DigitalOcean databases
  community.digitalocean.digital_ocean_database_info:
    oauth_token: "{{ lookup('ansible.builtin.env', 'DO_API_KEY') }}"
  register: my_databases
"""


RETURN = r"""
data:
  description: List of DigitalOcean databases
  returned: success
  type: list
  sample: [
    {
      "connection": {
        "database": "",
        "host": "testdatabase1-do-user-3097135-0.b.db.ondigitalocean.com",
        "password": "REDACTED",
        "port": 25061,
        "protocol":"rediss",
        "ssl": true,
        "uri": "rediss://default:REDACTED@testdatabase1-do-user-3097135-0.b.db.ondigitalocean.com:25061",
        "user": "default"
      },
      "created_at": "2021-04-21T15:41:14Z",
      "db_names": null,
      "engine": "redis",
      "id": "37de10e4-808b-4f4b-b25f-7b5b3fd194ac",
      "maintenance_window": {
        "day": "monday",
        "hour": "11:33:47",
        "pending": false
      },
      "name": "testdatabase1",
      "num_nodes": 1,
      "private_connection": {
        "database": "",
        "host": "private-testdatabase1-do-user-3097135-0.b.db.ondigitalocean.com",
        "password": "REDACTED",
        "port": 25061,
        "protocol": "rediss",
        "ssl": true,
        "uri": "rediss://default:REDACTED@private-testdatabase1-do-user-3097135-0.b.db.ondigitalocean.com:25061",
        "user": "default"
      },
      "private_network_uuid": "0db3519b-9efc-414a-8868-8f2e6934688c",
      "region": "nyc1",
      "size": "db-s-1vcpu-1gb",
      "status": "online",
      "tags": null,
      "users": null,
      "version": "6"
    },
    ...
  ]
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
)


class DODatabaseInfo(object):
    def __init__(self, module):
        self.module = module
        self.rest = DigitalOceanHelper(module)
        # pop the oauth token so we don't include it in the POST data
        self.module.params.pop("oauth_token")
        self.id = None
        self.name = None

    def get_by_id(self, database_id):
        if database_id is None:
            return None
        response = self.rest.get("databases/{0}".format(database_id))
        json_data = response.json
        if response.status_code == 200:
            database = json_data.get("database", None)
            if database is not None:
                self.id = database.get("id", None)
                self.name = database.get("name", None)
            return json_data
        return None

    def get_by_name(self, database_name):
        if database_name is None:
            return None
        page = 1
        while page is not None:
            response = self.rest.get("databases?page={0}".format(page))
            json_data = response.json
            if response.status_code == 200:
                for database in json_data["databases"]:
                    if database.get("name", None) == database_name:
                        self.id = database.get("id", None)
                        self.name = database.get("name", None)
                        return {"database": database}
                if (
                    "links" in json_data
                    and "pages" in json_data["links"]
                    and "next" in json_data["links"]["pages"]
                ):
                    page += 1
                else:
                    page = None
        return None

    def get_database(self):
        json_data = self.get_by_id(self.module.params["id"])
        if not json_data:
            json_data = self.get_by_name(self.module.params["name"])
        return json_data

    def get_databases(self):
        all_databases = []
        page = 1
        while page is not None:
            response = self.rest.get("databases?page={0}".format(page))
            json_data = response.json
            if response.status_code == 200:
                databases = json_data.get("databases", None)
                if databases is not None and isinstance(databases, list):
                    all_databases.append(databases)
                if (
                    "links" in json_data
                    and "pages" in json_data["links"]
                    and "next" in json_data["links"]["pages"]
                ):
                    page += 1
                else:
                    page = None
        return {"databases": all_databases}


def run(module):
    id = module.params.get("id", None)
    name = module.params.get("name", None)

    database = DODatabaseInfo(module)

    if id is not None or name is not None:
        the_database = database.get_database()
        if the_database:  # Found it
            module.exit_json(changed=False, data=the_database)
        else:  # Didn't find it
            if id is not None and name is not None:
                module.fail_json(
                    change=False, msg="Database {0} ({1}) not found".format(id, name)
                )
            elif id is not None and name is None:
                module.fail_json(change=False, msg="Database {0} not found".format(id))
            elif id is None and name is not None:
                module.fail_json(
                    change=False, msg="Database {0} not found".format(name)
                )
    else:
        all_databases = database.get_databases()
        module.exit_json(changed=False, data=all_databases)


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        id=dict(type="int", aliases=["database_id"]),
        name=dict(type="str"),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    run(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017, Thom Wiggers  <ansible@thomwiggers.nl>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: openssl_dhparam
short_description: Generate OpenSSL Diffie-Hellman Parameters
description:
    - This module allows one to (re)generate OpenSSL DH-params.
    - This module uses file common arguments to specify generated file permissions.
    - "Please note that the module regenerates existing DH params if they do not
      match the module's options. If you are concerned that this could overwrite
      your existing DH params, consider using the O(backup) option."
    - The module can use the cryptography Python library, or the C(openssl) executable.
      By default, it tries to detect which one is available. This can be overridden
      with the O(select_crypto_backend) option.
requirements:
    - Either cryptography >= 2.0
    - Or OpenSSL binary C(openssl)
author:
    - Thom Wiggers (@thomwiggers)
extends_documentation_fragment:
    - ansible.builtin.files
    - community.crypto.attributes
    - community.crypto.attributes.files
attributes:
    check_mode:
        support: full
    diff_mode:
        support: none
    safe_file_operations:
        support: full
options:
    state:
        description:
            - Whether the parameters should exist or not,
              taking action if the state is different from what is stated.
        type: str
        default: present
        choices: [ absent, present ]
    size:
        description:
            - Size (in bits) of the generated DH-params.
        type: int
        default: 4096
    force:
        description:
            - Should the parameters be regenerated even it it already exists.
        type: bool
        default: false
    path:
        description:
            - Name of the file in which the generated parameters will be saved.
        type: path
        required: true
    backup:
        description:
            - Create a backup file including a timestamp so you can get the original
              DH params back if you overwrote them with new ones by accident.
        type: bool
        default: false
    select_crypto_backend:
        description:
            - Determines which crypto backend to use.
            - The default choice is V(auto), which tries to use C(cryptography) if available, and falls back to C(openssl).
            - If set to V(openssl), will try to use the OpenSSL C(openssl) executable.
            - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
        type: str
        default: auto
        choices: [ auto, cryptography, openssl ]
        version_added: "1.0.0"
    return_content:
        description:
            - If set to V(true), will return the (current or generated) DH parameter's content as RV(dhparams).
        type: bool
        default: false
        version_added: "1.0.0"
seealso:
    - module: community.crypto.x509_certificate
    - module: community.crypto.openssl_csr
    - module: community.crypto.openssl_pkcs12
    - module: community.crypto.openssl_privatekey
    - module: community.crypto.openssl_publickey
'''

EXAMPLES = r'''
- name: Generate Diffie-Hellman parameters with the default size (4096 bits)
  community.crypto.openssl_dhparam:
    path: /etc/ssl/dhparams.pem

- name: Generate DH Parameters with a different size (2048 bits)
  community.crypto.openssl_dhparam:
    path: /etc/ssl/dhparams.pem
    size: 2048

- name: Force regenerate an DH parameters if they already exist
  community.crypto.openssl_dhparam:
    path: /etc/ssl/dhparams.pem
    force: true
'''

RETURN = r'''
size:
    description: Size (in bits) of the Diffie-Hellman parameters.
    returned: changed or success
    type: int
    sample: 4096
filename:
    description: Path to the generated Diffie-Hellman parameters.
    returned: changed or success
    type: str
    sample: /etc/ssl/dhparams.pem
backup_file:
    description: Name of backup file created.
    returned: changed and if O(backup) is V(true)
    type: str
    sample: /path/to/dhparams.pem.2019-03-09@11:22~
dhparams:
    description: The (current or generated) DH params' content.
    returned: if O(state) is V(present) and O(return_content) is V(true)
    type: str
    version_added: "1.0.0"
'''

import abc
import os
import re
import tempfile
import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.crypto.plugins.module_utils.version import LooseVersion

from ansible_collections.community.crypto.plugins.module_utils.io import (
    load_file_if_exists,
    write_file,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.math import (
    count_bits,
)

MINIMAL_CRYPTOGRAPHY_VERSION = '2.0'

CRYPTOGRAPHY_IMP_ERR = None
try:
    import cryptography
    import cryptography.exceptions
    import cryptography.hazmat.backends
    import cryptography.hazmat.primitives.asymmetric.dh
    import cryptography.hazmat.primitives.serialization
    CRYPTOGRAPHY_VERSION = LooseVersion(cryptography.__version__)
except ImportError:
    CRYPTOGRAPHY_IMP_ERR = traceback.format_exc()
    CRYPTOGRAPHY_FOUND = False
else:
    CRYPTOGRAPHY_FOUND = True


class DHParameterError(Exception):
    pass


class DHParameterBase(object):

    def __init__(self, module):
        self.state = module.params['state']
        self.path = module.params['path']
        self.size = module.params['size']
        self.force = module.params['force']
        self.changed = False
        self.return_content = module.params['return_content']

        self.backup = module.params['backup']
        self.backup_file = None

    @abc.abstractmethod
    def _do_generate(self, module):
        """Actually generate the DH params."""
        pass

    def generate(self, module):
        """Generate DH params."""
        changed = False

        # only generate when necessary
        if self.force or not self._check_params_valid(module):
            self._do_generate(module)
            changed = True

        # fix permissions (checking force not necessary as done above)
        if not self._check_fs_attributes(module):
            # Fix done implicitly by
            # AnsibleModule.set_fs_attributes_if_different
            changed = True

        self.changed = changed

    def remove(self, module):
        if self.backup:
            self.backup_file = module.backup_local(self.path)
        try:
            os.remove(self.path)
            self.changed = True
        except OSError as exc:
            module.fail_json(msg=to_native(exc))

    def check(self, module):
        """Ensure the resource is in its desired state."""
        if self.force:
            return False
        return self._check_params_valid(module) and self._check_fs_attributes(module)

    @abc.abstractmethod
    def _check_params_valid(self, module):
        """Check if the params are in the correct state"""
        pass

    def _check_fs_attributes(self, module):
        """Checks (and changes if not in check mode!) fs attributes"""
        file_args = module.load_file_common_arguments(module.params)
        if module.check_file_absent_if_check_mode(file_args['path']):
            return False
        return not module.set_fs_attributes_if_different(file_args, False)

    def dump(self):
        """Serialize the object into a dictionary."""

        result = {
            'size': self.size,
            'filename': self.path,
            'changed': self.changed,
        }
        if self.backup_file:
            result['backup_file'] = self.backup_file
        if self.return_content:
            content = load_file_if_exists(self.path, ignore_errors=True)
            result['dhparams'] = content.decode('utf-8') if content else None

        return result


class DHParameterAbsent(DHParameterBase):

    def __init__(self, module):
        super(DHParameterAbsent, self).__init__(module)

    def _do_generate(self, module):
        """Actually generate the DH params."""
        pass

    def _check_params_valid(self, module):
        """Check if the params are in the correct state"""
        pass


class DHParameterOpenSSL(DHParameterBase):

    def __init__(self, module):
        super(DHParameterOpenSSL, self).__init__(module)
        self.openssl_bin = module.get_bin_path('openssl', True)

    def _do_generate(self, module):
        """Actually generate the DH params."""
        # create a tempfile
        fd, tmpsrc = tempfile.mkstemp()
        os.close(fd)
        module.add_cleanup_file(tmpsrc)  # Ansible will delete the file on exit
        # openssl dhparam -out <path> <bits>
        command = [self.openssl_bin, 'dhparam', '-out', tmpsrc, str(self.size)]
        rc, dummy, err = module.run_command(command, check_rc=False)
        if rc != 0:
            raise DHParameterError(to_native(err))
        if self.backup:
            self.backup_file = module.backup_local(self.path)
        try:
            module.atomic_move(tmpsrc, self.path)
        except Exception as e:
            module.fail_json(msg="Failed to write to file %s: %s" % (self.path, str(e)))

    def _check_params_valid(self, module):
        """Check if the params are in the correct state"""
        command = [self.openssl_bin, 'dhparam', '-check', '-text', '-noout', '-in', self.path]
        rc, out, err = module.run_command(command, check_rc=False)
        result = to_native(out)
        if rc != 0:
            # If the call failed the file probably does not exist or is
            # unreadable
            return False
        # output contains "(xxxx bit)"
        match = re.search(r"Parameters:\s+\((\d+) bit\).*", result)
        if not match:
            return False  # No "xxxx bit" in output

        bits = int(match.group(1))

        # if output contains "WARNING" we've got a problem
        if "WARNING" in result or "WARNING" in to_native(err):
            return False

        return bits == self.size


class DHParameterCryptography(DHParameterBase):

    def __init__(self, module):
        super(DHParameterCryptography, self).__init__(module)
        self.crypto_backend = cryptography.hazmat.backends.default_backend()

    def _do_generate(self, module):
        """Actually generate the DH params."""
        # Generate parameters
        params = cryptography.hazmat.primitives.asymmetric.dh.generate_parameters(
            generator=2,
            key_size=self.size,
            backend=self.crypto_backend,
        )
        # Serialize parameters
        result = params.parameter_bytes(
            encoding=cryptography.hazmat.primitives.serialization.Encoding.PEM,
            format=cryptography.hazmat.primitives.serialization.ParameterFormat.PKCS3,
        )
        # Write result
        if self.backup:
            self.backup_file = module.backup_local(self.path)
        write_file(module, result)

    def _check_params_valid(self, module):
        """Check if the params are in the correct state"""
        # Load parameters
        try:
            with open(self.path, 'rb') as f:
                data = f.read()
            params = cryptography.hazmat.primitives.serialization.load_pem_parameters(data, backend=self.crypto_backend)
        except Exception as dummy:
            return False
        # Check parameters
        bits = count_bits(params.parameter_numbers().p)
        return bits == self.size


def main():
    """Main function"""

    module = AnsibleModule(
        argument_spec=dict(
            state=dict(type='str', default='present', choices=['absent', 'present']),
            size=dict(type='int', default=4096),
            force=dict(type='bool', default=False),
            path=dict(type='path', required=True),
            backup=dict(type='bool', default=False),
            select_crypto_backend=dict(type='str', default='auto', choices=['auto', 'cryptography', 'openssl']),
            return_content=dict(type='bool', default=False),
        ),
        supports_check_mode=True,
        add_file_common_args=True,
    )

    base_dir = os.path.dirname(module.params['path']) or '.'
    if not os.path.isdir(base_dir):
        module.fail_json(
            name=base_dir,
            msg="The directory '%s' does not exist or the file is not a directory" % base_dir
        )

    if module.params['state'] == 'present':
        backend = module.params['select_crypto_backend']
        if backend == 'auto':
            # Detection what is possible
            can_use_cryptography = CRYPTOGRAPHY_FOUND and CRYPTOGRAPHY_VERSION >= LooseVersion(MINIMAL_CRYPTOGRAPHY_VERSION)
            can_use_openssl = module.get_bin_path('openssl', False) is not None

            # First try cryptography, then OpenSSL
            if can_use_cryptography:
                backend = 'cryptography'
            elif can_use_openssl:
                backend = 'openssl'

            # Success?
            if backend == 'auto':
                module.fail_json(msg=("Cannot detect either the required Python library cryptography (>= {0}) "
                                      "or the OpenSSL binary openssl").format(MINIMAL_CRYPTOGRAPHY_VERSION))

        if backend == 'openssl':
            dhparam = DHParameterOpenSSL(module)
        elif backend == 'cryptography':
            if not CRYPTOGRAPHY_FOUND:
                module.fail_json(msg=missing_required_lib('cryptography >= {0}'.format(MINIMAL_CRYPTOGRAPHY_VERSION)),
                                 exception=CRYPTOGRAPHY_IMP_ERR)
            dhparam = DHParameterCryptography(module)

        if module.check_mode:
            result = dhparam.dump()
            result['changed'] = module.params['force'] or not dhparam.check(module)
            module.exit_json(**result)

        try:
            dhparam.generate(module)
        except DHParameterError as exc:
            module.fail_json(msg=to_native(exc))
    else:
        dhparam = DHParameterAbsent(module)

        if module.check_mode:
            result = dhparam.dump()
            result['changed'] = os.path.exists(module.params['path'])
            module.exit_json(**result)

        if os.path.exists(module.params['path']):
            try:
                dhparam.remove(module)
            except Exception as exc:
                module.fail_json(msg=to_native(exc))

    result = dhparam.dump()

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: devices_cellular_sims
short_description: Resource module for devices _cellular _sims
description:
- Manage operation update of the resource devices _cellular _sims.
- Updates the SIM and APN configurations for a cellular device.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  serial:
    description: Serial path parameter.
    type: str
  simFailover:
    description: SIM Failover settings.
    suboptions:
      enabled:
        description: Failover to secondary SIM (optional).
        type: bool
    type: dict
  sims:
    description: List of SIMs. If a SIM was previously configured and not specified
      in this request, it will remain unchanged.
    elements: dict
    suboptions:
      apns:
        description: APN configurations. If empty, the default APN will be used.
        elements: dict
        suboptions:
          allowedIpTypes:
            description: IP versions to support (permitted values include 'ipv4', 'ipv6').
            elements: str
            type: list
          authentication:
            description: APN authentication configurations.
            suboptions:
              password:
                description: APN password, if type is set (if APN password is not supplied,
                  the password is left unchanged).
                type: str
              type:
                description: APN auth type.
                type: str
              username:
                description: APN username, if type is set.
                type: str
            type: dict
          name:
            description: APN name.
            type: str
        type: list
      isPrimary:
        description: If true, this SIM is used for boot. Must be true on single-sim
          devices.
        type: bool
      slot:
        description: SIM slot being configured. Must be 'sim1' on single-sim devices.
        type: str
    type: list
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for devices updateDeviceCellularSims
  description: Complete reference of the updateDeviceCellularSims API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-device-cellular-sims
notes:
  - SDK Method used are
    devices.Devices.update_device_cellular_sims,

  - Paths used are
    put /devices/{serial}/cellular/sims,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_cellular_sims:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    apns:
    - allowedIpTypes:
      - ipv4
      - ipv6
      authentication:
        password: secret
        type: pap
        username: milesmeraki
      name: internet
    isPrimary: true
    serial: string
    slot: sim1

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_switch_stacks_routing_interfaces_dhcp
short_description: Resource module for networks _switch _stacks _routing _interfaces _dhcp
description:
- Manage operation update of the resource networks _switch _stacks _routing _interfaces _dhcp.
- Update a layer 3 interface DHCP configuration for a switch stack.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  bootFileName:
    description: The PXE boot server file name for the DHCP server running on the switch
      stack interface.
    type: str
  bootNextServer:
    description: The PXE boot server IP for the DHCP server running on the switch stack
      interface.
    type: str
  bootOptionsEnabled:
    description: Enable DHCP boot options to provide PXE boot options configs for the
      dhcp server running on the switch stack interface.
    type: bool
  dhcpLeaseTime:
    description: The DHCP lease time config for the dhcp server running on switch stack
      interface ('30 minutes', '1 hour', '4 hours', '12 hours', '1 day' or '1 week').
    type: str
  dhcpMode:
    description: The DHCP mode options for the switch stack interface ('dhcpDisabled',
      'dhcpRelay' or 'dhcpServer').
    type: str
  dhcpOptions:
    description: Array of DHCP options consisting of code, type and value for the DHCP
      server running on the switch stack interface.
    elements: dict
    suboptions:
      code:
        description: The code for DHCP option which should be from 2 to 254.
        type: str
      type:
        description: The type of the DHCP option which should be one of ('text', 'ip',
          'integer' or 'hex').
        type: str
      value:
        description: The value of the DHCP option.
        type: str
    type: list
  dhcpRelayServerIps:
    description: The DHCP relay server IPs to which DHCP packets would get relayed for
      the switch stack interface.
    elements: str
    type: list
  dnsCustomNameservers:
    description: The DHCP name server IPs when DHCP name server option is 'custom'.
    elements: str
    type: list
  dnsNameserversOption:
    description: The DHCP name server option for the dhcp server running on the switch
      stack interface ('googlePublicDns', 'openDns' or 'custom').
    type: str
  fixedIpAssignments:
    description: Array of DHCP fixed IP assignments for the DHCP server running on the
      switch stack interface.
    elements: dict
    suboptions:
      ip:
        description: The IP address of the client which has fixed IP address assigned
          to it.
        type: str
      mac:
        description: The MAC address of the client which has fixed IP address.
        type: str
      name:
        description: The name of the client which has fixed IP address.
        type: str
    type: list
  interfaceId:
    description: InterfaceId path parameter. Interface ID.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  reservedIpRanges:
    description: Array of DHCP reserved IP assignments for the DHCP server running on
      the switch stack interface.
    elements: dict
    suboptions:
      comment:
        description: The comment for the reserved IP range.
        type: str
      end:
        description: The ending IP address of the reserved IP range.
        type: str
      start:
        description: The starting IP address of the reserved IP range.
        type: str
    type: list
  switchStackId:
    description: SwitchStackId path parameter. Switch stack ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for switch updateNetworkSwitchStackRoutingInterfaceDhcp
  description: Complete reference of the updateNetworkSwitchStackRoutingInterfaceDhcp API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-stack-routing-interface-dhcp
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_stack_routing_interface_dhcp,

  - Paths used are
    put /networks/{networkId}/switch/stacks/{switchStackId}/routing/interfaces/{interfaceId}/dhcp,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_stacks_routing_interfaces_dhcp:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    bootFileName: home_boot_file
    bootNextServer: 1.2.3.4
    bootOptionsEnabled: true
    dhcpLeaseTime: 1 day
    dhcpMode: dhcpServer
    dhcpOptions:
    - code: '5'
      type: text
      value: five
    dnsCustomNameservers:
    - 8.8.8.8, 8.8.4.4
    dnsNameserversOption: custom
    fixedIpAssignments:
    - ip: 192.168.1.12
      mac: 22:33:44:55:66:77
      name: Cisco Meraki valued client
    interfaceId: string
    networkId: string
    reservedIpRanges:
    - comment: A reserved IP range
      end: 192.168.1.10
      start: 192.168.1.1
    switchStackId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_wireless_failed_connections_info
short_description: Information module for networks _wireless _failedconnections
description:
- Get all networks _wireless _failedconnections.
- List of all failed client connection events on this network in a given time range.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
    - NetworkId path parameter. Network ID.
    type: str
  t0:
    description:
    - T0 query parameter. The beginning of the timespan for the data. The maximum lookback period is 180 days from today.
    type: str
  t1:
    description:
    - T1 query parameter. The end of the timespan for the data. T1 can be a maximum of 7 days after t0.
    type: str
  timespan:
    description:
    - >
      Timespan query parameter. The timespan for which the information will be fetched. If specifying timespan, do
      not specify parameters t0 and t1. The value must be in seconds and be less than or equal to 7 days.
    type: float
  band:
    description:
    - >
      Band query parameter. Filter results by band (either '2.4', '5' or '6'). Note that data prior to February
      2020 will not have band information.
    type: str
  ssid:
    description:
    - Ssid query parameter. Filter results by SSID.
    type: int
  vlan:
    description:
    - Vlan query parameter. Filter results by VLAN.
    type: int
  apTag:
    description:
    - ApTag query parameter. Filter results by AP Tag.
    type: str
  serial:
    description:
    - Serial query parameter. Filter by AP.
    type: str
  clientId:
    description:
    - ClientId query parameter. Filter by client MAC.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for wireless getNetworkWirelessFailedConnections
  description: Complete reference of the getNetworkWirelessFailedConnections API.
  link: https://developer.cisco.com/meraki/api-v1/#!get-network-wireless-failed-connections
notes:
  - SDK Method used are
    wireless.Wireless.get_network_wireless_failed_connections,

  - Paths used are
    get /networks/{networkId}/wireless/failedConnections,
"""

EXAMPLES = r"""
- name: Get all networks _wireless _failedconnections
  cisco.meraki.networks_wireless_failed_connections_info:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    t0: string
    t1: string
    timespan: 0
    band: string
    ssid: 0
    vlan: 0
    apTag: string
    serial: string
    clientId: string
    networkId: string
  register: result

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "ssidNumber": 0,
        "vlan": 0,
        "clientMac": "string",
        "serial": "string",
        "failureStep": "string",
        "type": "string",
        "ts": "string"
      }
    ]
"""



# -*- coding: utf-8 -*-

# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

"""Provide version object to compare version numbers."""

from __future__ import absolute_import, division, print_function
__metaclass__ = type


# Once we drop support for ansible-core 2.11, we can remove the try/except.

from ansible.module_utils.six import raise_from

try:
    from ansible.module_utils.compat.version import LooseVersion, StrictVersion  # noqa: F401, pylint: disable=unused-import
except ImportError:
    try:
        from distutils.version import LooseVersion, StrictVersion  # noqa: F401, pylint: disable=unused-import
    except ImportError as exc:
        msg = 'To use this plugin or module with ansible-core 2.11, you need to use Python < 3.12 with distutils.version present'
        raise_from(ImportError(msg), exc)



#!/usr/bin/python
#
# Copyright (c) 2022 Andrea Decorte, <adecorte@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = """
---
module: azure_rm_natgateway
short_description: Manage Azure NAT Gateway instance
description:
    - Create, update and delete instances of Azure NAT Gateway.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    name:
        description:
            - The name of the NAT Gateway.
        required: True
        type: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    zones:
        description:
            - List of Availability Zones in which this NAT Gateway should be located.
        type: list
        elements: int
        choices:
            - 1
            - 2
            - 3
    sku:
        description:
            - SKU of the NAT gateway resource.
        type: dict
        suboptions:
            name:
                description:
                    - Name of the NAT gateway SKU. Defaults to C(standard).
                choices:
                    - 'standard'
                default: 'standard'
                type: str
    idle_timeout_in_minutes:
        description:
            - The idle timeout in minutes which should be used. Defaults to 4.
        default: 4
        type: int
    public_ip_addresses:
        description:
            - A list of Public IP Addresses which should be associated with the NAT Gateway resource.
            - Each element can be the name or resource id, or a dict contains C(name), C(resource_group) information of the IP address.
        type: list
        elements: str
    state:
        description:
            - Assert the state of the NAT gateway. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Andrea Decorte (@andreadecorte)

"""

EXAMPLES = """
- name: Create instance of NAT Gateway
  azure_rm_natgateway:
    resource_group: myResourceGroup
    name: myNATGateway
    public_ip_addresses:
      - pip_name

- name: Create instance of NAT Gateway
  azure_rm_natgateway:
    resource_group: myResourceGroup
    name: myNATGateway
    idle_timeout_in_minutes: 10
    location: eastus
    zones: [1]
    sku:
      name: standard
"""

RETURN = """
id:
    description:
        - NAT gateway resource ID.
    returned: always
    type: str
    sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/natGateways/myNATGw
name:
    description:
        - Name of NAT gateway.
    returned: always
    type: str
    sample: myNATGw
resource_group:
    description:
        - Name of resource group.
    returned: always
    type: str
    sample: myResourceGroup
location:
    description:
        - Location of NAT gateway.
    returned: always
    type: str
    sample: centralus
"""

import time
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, format_resource_id

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
    from azure.mgmt.core.tools import parse_resource_id, is_valid_resource_id
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


sku_spec = dict(
    name=dict(type='str', choices=['standard'], default='standard')
)


class AzureRMNATGateways(AzureRMModuleBase):
    """Configuration class for an Azure RM NAT Gateway resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            sku=dict(
                type='dict',
                options=sku_spec
            ),
            idle_timeout_in_minutes=dict(
                type='int',
                default=4
            ),
            zones=dict(
                type='list',
                elements='int',
                choices=[1, 2, 3]
            ),
            public_ip_addresses=dict(
                type='list',
                elements='str'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.parameters = dict()

        self.results = dict(changed=False)
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMNATGateways, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True, supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == "id":
                    self.parameters["id"] = kwargs[key]
                elif key == "location":
                    self.parameters["location"] = kwargs[key]
                elif key == "idle_timeout_in_minutes":
                    self.parameters["idle_timeout_in_minutes"] = kwargs[key]
                elif key == "zones":
                    self.parameters["zones"] = kwargs[key]
                elif key == "public_ip_addresses":
                    if "public_ip_addresses" not in self.parameters:
                        self.parameters["public_ip_addresses"] = []
                    for resource in kwargs[key]:
                        self.parameters["public_ip_addresses"].append({"id": self.return_resource_id(resource)})
                elif key == "sku":
                    ev = kwargs[key]
                    if "name" in ev:
                        if ev["name"] == "standard":
                            ev["name"] = "Standard"
                    self.parameters["sku"] = ev

        old_response = None
        response = None

        resource_group = self.get_resource_group(self.resource_group)

        if "location" not in self.parameters:
            self.parameters["location"] = resource_group.location

        # sku can be null, define a default value
        if "sku" not in self.parameters:
            self.parameters["sku"] = {"name": "Standard"}

        old_response = self.get_natgateway()

        if not old_response:
            self.log("NAT Gateway instance doesn't exist")
            if self.state == "absent":
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("NAT Gateway instance already exists")
            if self.state == "absent":
                self.to_do = Actions.Delete
            elif self.state == "present":
                self.log("Need to check if NAT Gateway instance has to be deleted or may be updated")
                self.to_do = Actions.Update

        if (self.to_do == Actions.Update):
            if (self.parameters["location"] != old_response["location"] or
                self.check_if_changed("zones", old_response) or
                self.check_if_changed("idle_timeout_in_minutes", old_response) or
                self.check_if_changed("public_ip_addresses", old_response) or
                    self.parameters["sku"]["name"] != old_response["sku"]["name"]):
                self.to_do = Actions.Update
            else:
                self.to_do = Actions.NoAction

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the NAT Gateway instance")

            if self.check_mode:
                self.results["changed"] = True
                self.results["parameters"] = self.parameters
                return self.results

            response = self.create_update_natgateway()

            if not old_response:
                self.results["changed"] = True
            else:
                self.results["changed"] = (old_response != response)
            self.log("Creation / Update done")

        elif self.to_do == Actions.Delete:
            self.log("NAT Gateway instance deleted")
            self.results["changed"] = True

            if self.check_mode:
                return self.results

            self.delete_natgateway()
            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_natgateway():
                time.sleep(20)
        else:
            self.log("NAT Gateway instance unchanged")
            self.results["changed"] = False
            response = old_response

        if response:
            self.results.update(self.format_response(response))

        return self.results

    def create_update_natgateway(self):
        """
        Creates or updates NAT Gateway with the specified configuration.

        :return: deserialized NAT Gateway instance state dictionary
        """
        self.log("Creating / Updating the NAT Gateway instance {0}".format(self.name))

        try:
            response = self.network_client.nat_gateways.begin_create_or_update(resource_group_name=self.resource_group,
                                                                               nat_gateway_name=self.name,
                                                                               parameters=self.parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log("Error attempting to create the NAT Gateway instance.")
            self.fail("Error creating the NAT Gateway instance: {0}".format(str(exc)))
        return response.as_dict()

    def delete_natgateway(self):
        """
        Deletes specified NAT Gateway instance in the specified subscription and resource group.

        :return: True
        """
        self.log("Deleting the NAT Gateway instance {0}".format(self.name))
        try:
            response = self.network_client.nat_gateways.begin_delete(resource_group_name=self.resource_group,
                                                                     nat_gateway_name=self.name)
        except Exception as e:
            self.log("Error attempting to delete the NAT Gateway instance.")
            self.fail("Error deleting the NAT Gateway instance: {0}".format(str(e)))

        return True

    def get_natgateway(self):
        """
        Gets the properties of the specified NAT Gateway.

        :return: deserialized NAT Gateway instance state dictionary
        """
        self.log("Checking if the NAT Gateway instance {0} is present".format(self.name))
        found = False
        try:
            response = self.network_client.nat_gateways.get(resource_group_name=self.resource_group,
                                                            nat_gateway_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("NAT Gateway instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log("Did not find the NAT Gateway instance.")
        if found is True:
            return response.as_dict()

        return False

    def check_if_changed(self, parameter_name, old_response):
        """"
        Compute if there is an update to the resource or not

        :return: True if resource is changed compared to the current one
        """
        if parameter_name in self.parameters and (parameter_name not in old_response or self.parameters[parameter_name] != old_response[parameter_name]):
            # Parameter changed
            return True
        elif parameter_name not in self.parameters and parameter_name in old_response:
            # Parameter omitted while it was specified before
            return True
        else:
            return False

    def format_response(self, natgw_dict):
        """
        Build format of the response

        :return dictionary filled with resource data
        """
        id = natgw_dict.get("id")
        id_dict = parse_resource_id(id)
        d = {
            "id": id,
            "name": natgw_dict.get("name"),
            "resource_group": id_dict.get("resource_group", self.resource_group),
            "location": natgw_dict.get("location")
        }
        return d

    def return_resource_id(self, resource):
        """
        Build an IP Address resource id from different inputs

        :return string containing the Azure id of the resource
        """
        if is_valid_resource_id(resource):
            return resource
        resource_dict = self.parse_resource_to_dict(resource)
        return format_resource_id(val=resource_dict["name"],
                                  subscription_id=resource_dict.get("subscription_id"),
                                  namespace="Microsoft.Network",
                                  types="publicIPAddresses",
                                  resource_group=resource_dict.get("resource_group"))


def main():
    """Main execution"""
    AzureRMNATGateways()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_vtp_version
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages VTP version configuration.
description:
- Manages VTP version configuration.
version_added: 1.0.0
author:
- Gabriele Gerbino (@GGabriele)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- VTP feature must be active on the device to use this module.
- This module is used to manage only VTP version.
- Use this in combination with M(cisco.nxos.nxos_vtp_password) and M(cisco.nxos.nxos_vtp_version)
  to fully manage VTP operations.
options:
  version:
    description:
    - VTP version number.
    required: true
    choices:
    - '1'
    - '2'
    - '3'
    type: str
"""
EXAMPLES = """
# ENSURE VTP VERSION IS 2
- cisco.nxos.nxos_vtp_version:
    version: 2
"""

RETURN = """
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"version": "2"}
existing:
    description:
        - k/v pairs of existing vtp
    returned: always
    type: dict
    sample: {"domain": "testing", "version": "1", "vtp_password": "password"}
end_state:
    description: k/v pairs of vtp after module execution
    returned: always
    type: dict
    sample: {"domain": "testing", "version": "2", "vtp_password": "password"}
updates:
    description: command sent to the device
    returned: always
    type: list
    sample: ["vtp version 2"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
"""
import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_capabilities,
    load_config,
    run_commands,
)


def execute_show_command(command, module, output="json"):
    cmds = [{"command": command, "output": output}]
    body = run_commands(module, cmds)
    return body


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_vtp_config(module):
    command = "show vtp status"
    body = execute_show_command(command, module, "text")[0]
    vtp_parsed = {}

    if body:
        version_regex = r".*VTP version running\s+:\s+(?P<version>\d).*"
        domain_regex = r".*VTP Domain Name\s+:\s+(?P<domain>\S+).*"

        try:
            match_version = re.match(version_regex, body, re.DOTALL)
            version = match_version.groupdict()["version"]
        except AttributeError:
            version = ""

        try:
            match_domain = re.match(domain_regex, body, re.DOTALL)
            domain = match_domain.groupdict()["domain"]
        except AttributeError:
            domain = ""

        if domain and version:
            vtp_parsed["domain"] = domain
            vtp_parsed["version"] = version
            vtp_parsed["vtp_password"] = get_vtp_password(module)

    return vtp_parsed


def get_vtp_password(module):
    command = "show vtp password"
    output = "json"
    cap = get_capabilities(module)["device_info"]["network_os_model"]
    if re.search(r"Nexus 6", cap):
        output = "text"

    body = execute_show_command(command, module, output)[0]

    if output == "json":
        password = body.get("passwd", "")
    else:
        password = ""
        rp = r"VTP Password: (\S+)"
        mo = re.search(rp, body)
        if mo:
            password = mo.group(1)

    return str(password)


def main():
    argument_spec = dict(version=dict(type="str", choices=["1", "2", "3"], required=True))

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()

    version = module.params["version"]

    existing = get_vtp_config(module)
    end_state = existing

    args = dict(version=version)

    changed = False
    proposed = dict((k, v) for k, v in args.items() if v is not None)
    delta = dict(set(proposed.items()).difference(existing.items()))

    commands = []
    if delta:
        commands.append(["vtp version {0}".format(version)])

    cmds = flatten_list(commands)
    if cmds:
        if module.check_mode:
            module.exit_json(changed=True, commands=cmds)
        else:
            changed = True
            load_config(module, cmds)
            end_state = get_vtp_config(module)
            if "configure" in cmds:
                cmds.pop(0)

    results = {}
    results["proposed"] = proposed
    results["existing"] = existing
    results["end_state"] = end_state
    results["updates"] = cmds
    results["changed"] = changed
    results["warnings"] = warnings

    module.exit_json(**results)


if __name__ == "__main__":
    main()
